//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** CPCTelera includes */
#include <cpctelera.h>
#include <stdio.h>

/** Structures includes */
#include "../structures/structures.h"

/** Decors Sprites */
#include "../sprites/allSprites.h"

/** Player include */
#include "player.h"
#include "playerMove.h"
#include "playerDraw.h"

/** Monster includes */
#include "../monsters/monsterEyes.h"

/** Game includes */
#include "../video/video.h"
#include "../sound/sound.h"
#include "../input/input.h"
#include "../room/room.h"
#include "../room/roomDrawCastle.h"
#include "../room/roomDrawItems.h"
#include "../world/world.h"
#include "../ui/uiboard.h"
#include "../ui/uichest.h"
#include "../text/fontsHelper.h"
#include "../text/text.h"
#include "../tools/tools.h"
#include "../sprites/functions/packedSprites.h"

/** External variables */
extern u8 gTimer;
extern u8 gLighterUses;
extern u8 gColorSky;
extern volatile BOOL gDrawRoomInProgress;
extern SRoom* gCurrentRoom;

/** Local defines */
#define TEMPO_UPDATE_LIFE		255
#define TEMPO_UPDATE_TORCH		32

/*****************************************************/
/*													 */
/*	Set Crowbar solidity							 */
/*												     */
/*****************************************************/
void SetCrowBarSolidity()
{
    gCrowBarSolidity = CROWBAR_NB_USES;
}

/*****************************************************/
/*													 */
/*	Get object in hand								 */
/*												     */
/*****************************************************/
u8 GetObjectInHand(u8 hand)
{
    return gPlayer.inHand[hand];
}

/*****************************************************/
/*													 */
/*	Empty hand										 */
/*												     */
/*****************************************************/
void EmptyHand(u8 hand)
{
    gPlayer.inHand[hand] = NOTHING;
    DrawHand(hand);
}

/*****************************************************/
/*													 */
/*	Is object in hands								 */
/*												     */
/*****************************************************/
u8 IsObjectInHand(u8 itemId)
{
    if (gPlayer.inHand[HAND_RIGHT] == itemId)
        return HAND_RIGHT;

    if (gPlayer.inHand[HAND_LEFT] == itemId)
        return HAND_LEFT;

    return NOT_IN_HANDS;
}

/*****************************************************/
/*													 */
/*	Is torch in hands								 */
/*												     */
/*****************************************************/
BOOL IsTorchInHand()
{
    return (IsObjectInHand(TORCH) != NOT_IN_HANDS);
}

/*****************************************************/
/*													 */
/*	Find free inventory slot						 */
/*												     */
/*****************************************************/
i8 FindFreeInventoryIndex()
{
    u8* inv = gPlayer.inventory;

    u8 i;
    for (i = 0; i < MAX_INVENTORY; i++)
    {
        if (*inv++ == NOTHING)
            return i;
    }
    return -1;
}

/*****************************************************/
/*													 */
/*	Put object in inventory							 */
/*												     */
/*****************************************************/
i8 PutInInventory(u8 itemId)
{
    i8 freeIndex = FindFreeInventoryIndex();
    if (freeIndex != NOT_FREE_SLOT)
    {
        gPlayer.inventory[freeIndex] = itemId;
        DrawInventorySlot(freeIndex);

        if (itemId == SCEPTER)
            gAdventure |= ADV_SCEPTER_TAKEN;

        if (itemId == RING)
            gAdventure |= ADV_RING_TAKEN;

        if (itemId == CROWN)
            gAdventure |= ADV_CROWN_TAKEN;

        if (IsGameWin())
        {
            PrintTextId(TEXT_ITS_NIGHT);
            gColorSky = NIGHT_COLOR;
        }

        PlaySelect();
        return freeIndex;
    }

    PrintTextId(TEXT_NO_FREE_SLOT);
    return NOT_FREE_SLOT;
}

/*****************************************************/
/*													 */
/*	Put object in hand								 */
/*												     */
/*****************************************************/
BOOL PutInHand(u8 hand, u8 itemId)
{
    u8 inHand = gPlayer.inHand[hand];
    if (inHand != NOTHING)
    {		
        if (PutInInventory(inHand) == NOT_FREE_SLOT)
            return FALSE;
    }

    gPlayer.inHand[hand] = itemId;
    DrawHand(hand);
    PlaySelect();
    return TRUE;
}

/*****************************************************/
/*													 */
/*	Use crow bar									 */
/*												     */
/*****************************************************/
void UseCrowBar(BOOL opened)
{
    if (!opened)
        PrintTextId(TEXT_LOCK_RESIST);
 
    if (--gCrowBarSolidity == 0)
    {
        u8 hand = IsObjectInHand(CROWBAR);
        EmptyHand(hand);
        SetCrowBarSolidity();
        PrintTextId(TEXT_CROWBAR_BREAK);
    }

    PlayImpact();
}

/*****************************************************/
/*													 */
/*	Use object										 */
/*												     */
/*****************************************************/
BOOL UseObjectInInventory(u8 index)
{
    u8 ret = TRUE;

    u8 itemId = gPlayer.inventory[index];
    u8* itemPtr = &gPlayer.inventory[index];
    if (itemId == POTION)
    {
        UpdateLife(POTION_EXTRA_LIFE);
        DrawLife();
        PlaySelect();
        *itemPtr = NOTHING;
    }
    else if (itemId == LIGHTER)
    {
        if (IsTorchInHand() && !gTorchHand.lighted)
        {
            BOOL hasLight = HasLightInRoom();

            gTorchHand.lighted = TRUE;
            LightTorchInv();
            PlayFlame();

            if (!hasLight)
            {
                InitRoomVar();
                DrawRoom();
                SetupPlayerSpriteMove();
            }

            if (--gLighterUses == 0)
            {
                gLighterUses = LIGHTER_NB_USES;
                *itemPtr = NOTHING;
            }
        }
    }
    else if (itemId >= SCROLL)
    {
        u8 text = TEXT_SCROLL_0;

        if (itemId == SCROLL_1)
            text = TEXT_SCROLL_1;
        else
        if (itemId == SCROLL_2)
            text = TEXT_SCROLL_2;

        PrintTextId(text);
        PlaySelect();
        *itemPtr = NOTHING;
    }
    else
    {
        ret = FALSE;
    }

    return ret;
}

/*****************************************************/
/*													 */
/*	Use object in hand								 */
/*												     */
/*****************************************************/
void UseObjectInHand(u8 hand)
{
    u8 roomId = gRoomId;
    u8 dir = GetPlayerDir();
    u8 exitValue = CanExitDir(dir);
    BOOL inFrontOfDoorClosed = (exitValue == EXIT_DOOR_CLOSED);	

    u8 itemId = gPlayer.inHand[hand];

    if (itemId == METAL_KEY || itemId == GOLD_KEY)
    {
        if (inFrontOfDoorClosed)
            TryOpenLockedDoor(METAL_KEY);
    }
    if (itemId == GOLD_KEY)
    {
        TryOpenSecretPassage();
    }	
    else if (itemId == ROPE)
    {
        if (roomId == DUNGEON_PIT_ROOM)
        {
            if (dir == TOP)
                ExitRoom(TOP);
        }
        else
        if (roomId == PIT_ROOM)
        {
            ExitRoom(CLIMB_UP_PIT);
        }
        else
        {
            InteractPlayer(ROPE);
        }
    }
    else if (itemId == CROWBAR)
    {
        if (inFrontOfDoorClosed)
            TryOpenLockedDoor(CROWBAR);
        else
            InteractPlayer(CROWBAR);
    }
    else if (itemId == GOLDEN_CUP)
    {
        if (roomId == LAC_FOREST_ROOM && gPlayerPos.x <= GetLimitOfRoom(LIMIT_LEFT))
        {
            if (GetPlayerMovement() == CROUCH_DOWN)
            {
                gAdventure |= ADV_CUP_FILLED;
                DrawHand(HAND_LEFT);
                PrintTextId(TEXT_CUP_FILLED);
                PlaySelect();
            }
            else
                PrintTextId(TEXT_MUST_CROUCH);
        }
        else if (roomId == DEMON_ROOM && (gAdventure & ADV_CUP_FILLED))
        {            
            gAdventure |= ADV_DEMON_VANISHED;
            PrintTextId(TEXT_DEAMON_VANISHED);
            PlayFlame();
            DrawRoom();
            EmptyHand(hand);
        }
    }
}

/*****************************************************/
/*													 */
/*	Item interaction								 */
/*												     */
/*****************************************************/
void InteractWithItem(u8* itemPtr, u8 usedItem)
{	
    u8 itemId = *itemPtr;
    if (itemId != NOTHING)
    {
        if ((itemId == ROOM_CHEST_CLOSE || itemId == ROOM_CHEST_OPEN) && GetPlayerDir() == LEFT)
        {
            if (usedItem == CROWBAR || usedItem == NOTHING)
            {
                if (GetPlayerMovement() == CROUCH_DOWN)
                {
                    if (itemId == ROOM_CHEST_CLOSE)
                    {
                        BOOL isOpened = IsChestOpened();
                        if (!isOpened)
                        {
                            if (usedItem == CROWBAR)
                                isOpened = ForceChest();
                            else
                                PrintTextId(TEXT_LOCKED);
                        }

                        if (isOpened)
                        {
                            PrintTextId(TEXT_UNLOCKED);
                            *itemPtr = ROOM_CHEST_OPEN;
                            DrawChestOpened();
                            DrawChest();
                        }
                    }
                    else
                    {
                        if (!IsChestEmpty())
                            DrawChest();
                    }
                }
                else
                    PrintTextId(TEXT_MUST_CROUCH);
            }
        }
        else if (itemId == GOLD_PURSE)
        {
            if (usedItem == CHEST_ACTION || usedItem == NOTHING)
            {
                gPlayer.gold += (NB_GOLD_COINS + GetRandMax(10));
                DrawGold();
                PlaySelect();
                *itemPtr = NOTHING;

                if (usedItem == CHEST_ACTION)
                    return;
            }
        }
        else if (itemId == ROOM_ITEM_TRAP)
        {
            if (usedItem == ROPE)
                ExitRoom(CLIMB_DOWN_PIT);
        }        
        else if (itemId < OBJECTS)
        {
            if (usedItem == CHEST_ACTION || usedItem == NOTHING)
            {
                u8 hand = GetHandForObject(itemId);
                if (itemId <= HANDLE_OBJECTS && GetObjectInHand(hand) == NOTHING)
                {
                    PutInHand(hand, itemId);
                    *itemPtr = NOTHING;
                }
                else
                {
                    if (PutInInventory(itemId) != NOT_FREE_SLOT)
                        *itemPtr = NOTHING;
                }
            }
        }
        else
        if (usedItem == NOTHING)
        {
            if (itemId == WALL_FIREPLACE_KEY)
            {
                if (PutInHand(HAND_LEFT, METAL_KEY))
                    *itemPtr = WALL_FIREPLACE;
            }
            else if (itemId >= WALL_ALTAR_GOLD_CUP && itemId <= WALL_ALTAR_SCEPTER)
            {
                static const u8 sObjectsOnAltar[] = { GOLDEN_CUP, RING, CROWN, SCEPTER };
                u8 objectAltar = sObjectsOnAltar[itemId - WALL_ALTAR_GOLD_CUP];

                if (PutInInventory(objectAltar) != NOT_FREE_SLOT)
                    *itemPtr = WALL_ALTAR;               
            }
            else if (itemId == WALL_PAINT_3)
            {
                PrintTextId(TEXT_PERSIA);
            }        
        }

        if (itemId != *itemPtr)
            DrawInteractiveItems(FALSE);
    }
}

/*****************************************************/
/*													 */
/*	Player interaction								 */
/*												     */
/*****************************************************/
void InteractPlayer(u8 usedItem)
{
    u8* item = NULL;
    if (IsFrontTopWall())
    {
        u8 playerX = gPlayerPos.x + PLAYER_WIDTH / 2;
        u8 playerXCx = playerX  + PLAYER_WIDTH;

        SWidth* decoCenter = GetDecoCenterCoord();

        u8 i;
        for (i = 0; i < NB_ROOM_ITEMS; i++)
        {
            if (decoCenter->cx != 0)
            {
                if (playerX <= decoCenter->cx && playerXCx >= decoCenter->x)
                {
                    item = &gCurrentRoom->wallFront[i];
                    if (*item >= WALL_ALTAR_GOLD_CUP)
                        break;
                }
            }
            decoCenter++;
        }
    }

    if (item == NULL)
        item = GetCollideGroundItem();

    if (item != NULL)
        InteractWithItem(item, usedItem);
}

/*****************************************************/
/*													 */
/*	Extinguish torch								 */
/*												     */
/*****************************************************/
void ExtinguishTorch()
{    
    u8 lighted = gTorchHand.lighted;
    gTorchHand.lighted = FALSE;

    DrawHand(HAND_RIGHT);

    if (lighted)
    {
        PrintTextId(TEXT_TORCH_DOWN);
        SetRoomDarkness();
    }
}

/*****************************************************/
/*													 */
/*	No torch in hand								 */
/*												     */
/*****************************************************/
void SetNoTorchInHand()
{
    EmptyHand(HAND_RIGHT);
    gTorchHand.lighted = FALSE;
}

/*****************************************************/
/*													 */
/*	Update Torch									 */
/*												     */
/*****************************************************/
void UpdateTorch()
{
    if (IsTorchInHand() && gTorchHand.burnout > 0)
    {
        if (--gTorchHand.burnout == 0)
        {
            SetNoTorchInHand();
            PrintTextId(TEXT_TORCH_DOWN);
            SetRoomDarkness();            
        }
    }
}


/*****************************************************/
/*													 */
/*	Wait for restart game							 */
/*												     */
/*****************************************************/
void WaitForRestart()
{
    WaitForFire();

    InitPlayerGame();
    CloseBoard();
    RedrawBoard();

    ResetWorld();
    EnterRoom();
}

/*****************************************************/
/*													 */
/*	Update player									 */
/*												     */
/*****************************************************/
void UpdatePlayer()
{
    if (gRoomId == WIN_ROOM)
    {
        SetPlayerDir(DOWN);
        DrawPlayerStay();       

        if (CheckUserInput() == FIRE)
            WaitForRestart();
    }
    else
    {
        UpdatePlayerMove();
        
        if (!HasLightInRoom())
            DrawMonsterEyes();

        if (gTimer == TEMPO_UPDATE_LIFE && gRoomId > LAC_FOREST_ROOM)
            UpdateLife(-1);

        if (gPlayer.life == 0)
            PlayerDead();
    }

    if ((ModuloPowerTwo(gTimer, TEMPO_UPDATE_TORCH)) == 0)
        UpdateTorch();
}

/*****************************************************/
/*													 */
/*	Set player start position						 */
/*												     */
/*****************************************************/
void SetPlayerStartPos()
{
    InitRooms();

    gPlayerPos.x = START_POS_X;
    gPlayerPos.y = START_POS_Y;
    SetRoomPosY();
    SetPlayerDir(DOWN);

    InitPlayer();
}

/*****************************************************/
/*													 */
/*	Init player	game								 */
/*												     */
/*****************************************************/
void InitPlayerGame()
{
    gTorchHand.burnout = 0;
    cpct_memset(&gPlayer, NOTHING, sizeof(SPlayer));

    //gPlayer.inHand[HAND_RIGHT] = TORCH;
    //gPlayer.inHand[HAND_LEFT] = GOLD_KEY;

#ifdef DEBUG_MODE
    gTorchHand.lighted = TRUE;
#else
    gTorchHand.lighted = FALSE;
#endif

    SetCrowBarSolidity();
    gLighterUses = LIGHTER_NB_USES;

    gAdventure = ADV_START;

    gPlayer.gold = START_GOLD;
    gPlayer.life = MAX_LIFE;

    SetPlayerStartPos();
}

/*****************************************************/
/*													 */
/*	Player Dead										 */
/*												     */
/*****************************************************/
void PlayerDead()
{   
    u8* vmem = cpctm_screenPtr(CPCT_VMEM_START, (ROOM_CX - SKELETON_W) / 2, (ROOM_CY - SKELETON_H) / 2);

    gDrawRoomInProgress = TRUE;

    CleanRoom(ROOM_CY);
    DrawPackedSpriteMaskedFlip(G_SKELETON_INDEX, vmem, SKELETON_W, SKELETON_H, FALSE);

    PrintTextId(TEXT_DEAD);

    gDrawRoomInProgress = FALSE;

    WaitForRestart();
}

/*****************************************************/
/*													 */
/*	Update life										 */
/*												     */
/*****************************************************/
void UpdateLife(i8 upLife)
{
    i16 newLife = gPlayer.life + upLife;
    if (newLife > MAX_LIFE)
    {
        gPlayer.life = MAX_LIFE;
    }
    else if (newLife <= 0)
    {
        gPlayer.life = 0;
    }
    else
    {
        gPlayer.life = (u8)newLife;
    }	

    DrawLife();
}