//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** CPCTelera includes */
#include <cpctelera.h>

/** Structures includes */
#include "../structures/structures.h"
#include "../structures/memMapping.h"

/** Asm includes */
#include "../asm/shiftSpritePixel.h"

/** Game includes */
#include "../tools/tools.h"
#include "../video/video.h"
#include "../sprites/functions/moveSprite.h"
#include "../sprites/functions/packedSprites.h"
#include "../input/input.h"
#include "../room/room.h"
#include "../items/torchFlame.h"

/** Monster includes */
#include "../monsters/monsterEyes.h"

/** Sprite */
#include "../sprites/allSprites.h"

/** Player include */
#include "player.h"
#include "playerDraw.h"
#include "playerMove.h"

/** Position player */
#define POS_CHEST_Y				(FLAMES_CY + 9)
#define OFFSET_HAND				11
#define POS_HAND_LEFT_Y			(POS_CHEST_Y + 11)
#define POS_HAND_RIGHT_Y		(FLAMES_CY)
#define POS_LEGS_Y				(POS_CHEST_Y + LATT_CHEST_H + LATT_HEAD_CY)
    
#define POS_LAT_X				0
#define POS_FRONT_X				1

#define POS_CHEST_TOP_X			(POS_FRONT_X + 0)
#define POS_CHEST_DOWN_X		(POS_FRONT_X + 3)
#define POS_CHEST_LEFT_X		(POS_LAT_X + 3)
#define POS_CHEST_RIGHT_X		(POS_LAT_X + 2)

#define POS_HAND_TOP_X			(POS_CHEST_TOP_X + 6)
#define POS_HAND_DOWN_X			(POS_CHEST_DOWN_X - 2)
#define POS_HAND_LEFT_X			(POS_CHEST_LEFT_X - 2)
#define POS_HAND_RIGHT_X		(POS_CHEST_RIGHT_X + 6)

#define POS_LEGS_TOP_X			(POS_CHEST_TOP_X + 2)
#define POS_LEGS_DOWN_X			(POS_CHEST_DOWN_X)
#define POS_LEGS_LEFT_X			(POS_CHEST_LEFT_X)
#define POS_LEGS_RIGHT_X		(POS_CHEST_RIGHT_X)

#define POS_H_JUMP_TOP_X		(POS_LEGS_TOP_X - 2)
#define POS_H_JUMP_DOWN_X		(POS_LEGS_DOWN_X - 1)
#define POS_H_JUMP_LEFT_X		(POS_LEGS_LEFT_X)
#define POS_H_JUMP_RIGHT_X		(POS_LEGS_RIGHT_X)

#define CHEST_CY				LATT_CHEST_H
#define LEGS_CY					FRONT_LEGS_H
#define HAND_CY					HAND_EMPTY_H
#define HAND_CX					HAND_EMPTY_W

#define FIRST_FRONT_OFFSET		5
#define FRONT_OFFSET			7

#define POS_X_JUMP_LONG			0
#define POS_X_JUMP_LONG_FLIP	1
#define POS_Y_JUMP_LONG			(PLAYER_SPRITE_CY - LONG_JUMP_H)
#define POS_Y_JUMP_LONG_HEAD	(PLAYER_SPRITE_CY - LONG_JUMP_H - 9)

#define LATT_HEAD_CX			LATT_HEAD_W
#define LATT_HEAD_CY			(LATT_HEAD_H / 2)
#define LATT_HEAD_SZ			(LATT_HEAD_H * LATT_HEAD_W) / 2

/** Position player defined */
static const u8 _posChest[] =
{
    POS_CHEST_TOP_X,
    POS_CHEST_DOWN_X,
    POS_CHEST_LEFT_X,
    POS_CHEST_RIGHT_X
};

static const u8 _posHand[] =
{
    POS_HAND_TOP_X,
    POS_HAND_DOWN_X,
    POS_HAND_LEFT_X,
    POS_HAND_RIGHT_X
};

static const u8 _posLegs[] =
{
    POS_LEGS_TOP_X,
    POS_LEGS_DOWN_X,
    POS_LEGS_LEFT_X,
    POS_LEGS_RIGHT_X
};

static const u8 _posLegsHighJump[] =
{
    POS_H_JUMP_TOP_X,
    POS_H_JUMP_DOWN_X,
    POS_H_JUMP_LEFT_X,
    POS_H_JUMP_RIGHT_X
};

/** Player Sprite buffers */
u8* _spriteBuffer;
u8* _spritePlayer;
u8* _backgroundPlayer;

/** Player move */
SMoveSprite _playerMove;
u8 _walkStep;
u8* _longJumpDrawPtr;

/** Shared player variables */
extern SCoord gPlayerPos;
extern u8 _playerStepCycle;
extern u8 _playerMovement;
extern u8 _dir;

/*****************************************************/
/*													 */
/*	Get Player Sprite VMem							 */
/*												     */
/*****************************************************/
u8* GetScreenPlayer()
{
    return GetScreenPtr(gPlayerPos.x, gPlayerPos.y);
}

/*****************************************************/
/*													 */
/*	Convert direction to position					 */
/*												     */
/*****************************************************/
u8 GetDirToPos()
{
    if (HAS_MOVE_Y_AXIS(_dir))
        return _dir - 1;

    return (_dir == LEFT) ? 2 : 3;
}

/*****************************************************/
/*													 */
/*	Get player Background buffer					 */
/*												     */
/*****************************************************/
u8* GetPlayerBackgroundBuffer()
{
    return _backgroundPlayer;
}

/*****************************************************/
/*													 */
/*	Get Player Sprite buffer pointer	   			 */
/*												     */
/*****************************************************/
u8* GetPlayerSpriteBufferPtr(u8 x, u8 y)
{
    return cpctm_spriteBufferPtr(_spritePlayer, PLAYER_SPRITE_CX, x, y);
}

/*****************************************************/
/*													 */
/*	Initialize Player Graphics						 */
/*												     */
/*****************************************************/
void InitPlayerGraphics()
{
    _playerMove.backGround = _backgroundPlayer = _spriteBuffer = GET_PLAYER_LOC_START;
    _playerMove.sprite = _spritePlayer = _spriteBuffer + PLAYER_SPRITE_SZ;

    _longJumpDrawPtr = GetPlayerSpriteBufferPtr(POS_X_JUMP_LONG, POS_Y_JUMP_LONG);
}

/*****************************************************/
/*													 */
/*	Setup Player Sprite								 */
/*												     */
/*****************************************************/
void SetupPlayerSpriteMove()
{
    SetupMoveSprite(&_playerMove, &gPlayerPos);
}

/*****************************************************/
/*													 */
/*	Draw into sprite buffer of player    			 */
/*												     */
/*****************************************************/
void DrawIntoSpritePlayer(u8* playerBuffer, u8 width, u8 height, u8* sprite)
{
    cpct_drawToSpriteBufferMaskedAlignedTable(PLAYER_SPRITE_CX, playerBuffer, width, height, sprite, gMaskTable);
}

/*****************************************************/
/*													 */
/*	Draw into sprite buffer of player    			 */
/*												     */
/*****************************************************/
void DrawIntoSpritePlayerCompressedMirrored(u8* playerBuffer, u8 width, u8 height, u8 shift, u8* spriteCompressed)
{
    u8* sprite = Decompress(spriteCompressed);
    DrawIntoSpritePlayer(playerBuffer, width, height, sprite);

    FlipAndShiftSprite(sprite, width, height, shift);

    DrawIntoSpritePlayer(playerBuffer + width - 1, width, height, sprite);
}

/*****************************************************/
/*													 */
/*	Draw into sprite buffer of player    			 */
/*												     */
/*****************************************************/
void DrawIntoSpritePlayerCompressed(u8* playerBuffer, u8 width, u8 height, u8* spriteCompressed, BOOL flip)
{
    u8* sprite = GetDecompressSpriteFlip(spriteCompressed, width, height, flip);
    DrawIntoSpritePlayer(playerBuffer, width, height, sprite);
}

/*****************************************************/
/*													 */
/*	Draw Player Sprite Chest						 */
/*												     */
/*****************************************************/
void DrawPlayerChest(u8 y)
{
    u8 dir = _dir;	
    u8 x = _posChest[GetDirToPos()] + 1;
    u8* drawPtr;

    if (HAS_MOVE_X_AXIS(dir))
    {
        BOOL flip = (dir == RIGHT);
                
        drawPtr = GetPlayerSpriteBufferPtr(flip ? x + 2 : x, y);
        DrawIntoSpritePlayer(drawPtr, LATT_HEAD_CX, LATT_HEAD_CY, flip ? latt_head + LATT_HEAD_SZ : latt_head);

        drawPtr = GetPlayerSpriteBufferPtr(x, y + LATT_HEAD_CY);
        DrawIntoSpritePlayerCompressed(drawPtr, LATT_CHEST_W, LATT_CHEST_H, G_latt_chest, flip);
    }
    else
    {
        BOOL isDown = (dir == DOWN);

        u8* sprite = isDown ? front_head_0 : front_head_1;
        drawPtr = GetPlayerSpriteBufferPtr(isDown ? x : x + 2, y);
        DrawIntoSpritePlayer(drawPtr, FRONT_HEAD_0_W, FRONT_HEAD_0_H, sprite);

        sprite = isDown ? G_front_chest_0 : G_front_chest_1;
        drawPtr = GetPlayerSpriteBufferPtr(isDown ? x - 1: x, y + FRONT_HEAD_0_H);
        DrawIntoSpritePlayerCompressedMirrored(drawPtr, FRONT_CHEST_W, FRONT_CHEST_H, isDown ? LEFT : RIGHT, sprite);
    }

    if (gRoomId == WIN_ROOM)
    {
        drawPtr = GetPlayerSpriteBufferPtr(x, y - OBJECTS_H + 3);
        DrawIntoSpritePlayerCompressed(drawPtr, OBJECTS_W, OBJECTS_H, GetPackedSprite(G_OBJECTS_11_INDEX), FALSE);
    }
}

/*****************************************************/
/*													 */
/*	Get Player Sprite Hands							 */
/*												     */
/*****************************************************/
u8* GetPlayerSpriteHandRight()
{
    u8** hands;
    u8 index = HAS_MOVE_Y_AXIS(_dir) ? 0 : 1;
    
    if (IsTorchInHand())
        hands = (u8**)G_hand_torch_tileset;
    else
        hands = (u8**)G_hand_empty_tileset;

    return hands[index];
}

/*****************************************************/
/*													 */
/*	Get Hand position								 */
/*												     */
/*****************************************************/
u8 GetPosxHand()
{
    u8 pos = GetDirToPos();
    return _posHand[pos];
}

/*****************************************************/
/*													 */
/*	Get Crowbar position							 */
/*												     */
/*****************************************************/
u8 GetPosxCrowbar()
{
    u8 x = GetPosxHand() + 1;

    if (_dir & (DOWN | LEFT))
        x-= 2;

    return x;
}

/*****************************************************/
/*													 */
/*	Get torch position								 */
/*												     */
/*****************************************************/
u8 GetPosxTorch()
{
    u8 x = GetPosxHand();
    u8 dir = _dir;

    if (dir == TOP)
        x++;
    else 
    if (dir == DOWN)
        x--;
    
    return x;
}

/*****************************************************/
/*													 */
/*	Draw Player Torch Flame							 */
/*												     */
/*****************************************************/
void DrawPlayerTorchFlame(u8 yHand, u8 dir)
{
    #define FLAMES_BURNOUT	30
    #define FLAMES_POS		4

    u8* flame = GetFlameSprite(FLAME_PLAYER);
    
    u8 flamesBurn = FLAMES_POS - gTorchHand.burnout / FLAMES_BURNOUT;
    u8 offsety = HAS_MOVE_X_AXIS(dir) ? 1 : 0;
    u8 y = yHand - FLAMES_CY + flamesBurn + offsety;
    u8 x = GetPosxTorch() + ((dir & (TOP|RIGHT)) ? 2 : -1);

    u8* drawPtr = GetPlayerSpriteBufferPtr(x, y);
    DrawIntoSpritePlayer(drawPtr, FLAMES_CX, FLAMES_CY, flame);
}

/*****************************************************/
/*													 */
/*	Draw Player object in left hand					 */
/*												     */
/*****************************************************/
void DrawObjectInHandLeft(u8 posChestY)
{	
    #define POS_Y_KEY		29
    #define POS_Y_CALICE	24

    u8 dir = _dir;
    u8 leftObject = GetObjectInHand(HAND_LEFT);
    u8* drawPtr;

    /** Draw object in left hand */
    if (leftObject != NOTHING && dir != TOP)
    {
        BOOL flip = (dir == RIGHT && dir != DOWN);
        u8 yHandLeft = posChestY;
        u8 xHandLeft = 6;

        if (leftObject != GOLDEN_CUP)
        {
            yHandLeft += POS_Y_KEY;
            xHandLeft = 5;
            if (HAS_MOVE_X_AXIS(dir))
                xHandLeft = flip ? 8 : 1;

            drawPtr = GetPlayerSpriteBufferPtr(xHandLeft, yHandLeft);
            DrawIntoSpritePlayerCompressed(drawPtr, KEYS_W, KEYS_H, G_keys_tileset[leftObject == GOLD_KEY ? 0 : 1], flip);
        }
        else
        {
            yHandLeft += POS_Y_CALICE;
            xHandLeft = 7;
            if (HAS_MOVE_X_AXIS(dir))
                xHandLeft = flip ? 6 : 3;
            else
                yHandLeft++;

            drawPtr = GetPlayerSpriteBufferPtr(xHandLeft, yHandLeft);
            DrawIntoSpritePlayerCompressed(drawPtr, OBJECTS_W, OBJECTS_H, GetPackedSprite(G_OBJECTS_01_INDEX), FALSE);
        }
    }
}

/*****************************************************/
/*													 */
/*	Draw Player left hand							 */
/*												     */
/*****************************************************/
void DrawHandLeft(u8 posChestY)
{
    if (HAS_MOVE_Y_AXIS(_dir))
    {
        BOOL isDownDir = (_dir == DOWN);

        u8* drawPtr = GetPlayerSpriteBufferPtr(isDownDir ? HAND_EMPTY_H : 0, isDownDir ? posChestY + OFFSET_HAND - 1 : posChestY + OFFSET_HAND);
        u8* sprite = GetDecompressSpriteFlip(G_hand_empty_0, HAND_CX, HAND_CY, isDownDir);

        if (isDownDir)
            asm_shiftSpritePixelsLeft(sprite, HAND_EMPTY_SZ);
        else
            asm_shiftSpritePixelsRight(sprite, HAND_EMPTY_SZ);

        DrawIntoSpritePlayer(drawPtr, HAND_CX, HAND_CY, sprite);
    }

    DrawObjectInHandLeft(posChestY);
}

/*****************************************************/
/*													 */
/*	Draw CrowBar in right hand						 */
/*												     */
/*****************************************************/
void DrawCrowBar(u8 yHands, u8 dir)
{
    u8 xHandRight = GetPosxCrowbar();
    u8* sprite = (dir & (LEFT | RIGHT)) ? G_pied_biche_1 : G_pied_biche_0;
    BOOL flip = (dir & (LEFT | TOP));

    u8* drawPtr = GetPlayerSpriteBufferPtr(xHandRight, yHands + 16);
    DrawIntoSpritePlayerCompressed(drawPtr, PIED_BICHE_W, PIED_BICHE_H, sprite, flip);

    if (dir == DOWN)
    {
        drawPtr = GetPlayerSpriteBufferPtr(xHandRight + 4, yHands + 17);
        DrawIntoSpritePlayerCompressed(drawPtr, PIED_BICHE_W, PIED_BICHE_H, G_pied_biche_1, FALSE);
    }
}

/*****************************************************/
/*													 */
/*	Draw Rope in right hand							 */
/*												     */
/*****************************************************/
void DrawRope(u8 yHands, u8 dir)
{
    u8* drawPtr;

    u8 yHandRight = (dir & (LEFT | RIGHT)) ? yHands + 19 : yHands + 21;
    u8 xHandRight = GetPosxHand();
    if (dir == RIGHT)
    {
        xHandRight -= 2;
    }
    else if (dir == TOP)
    {
        xHandRight--;
    }
    else
    {
        xHandRight++;
    }

    drawPtr = GetPlayerSpriteBufferPtr(xHandRight, yHandRight);
    if (dir != TOP)
        DrawIntoSpritePlayerCompressed(drawPtr, ROPE_W, ROPE_H, G_rope_0, FALSE);

    DrawIntoSpritePlayerCompressed(drawPtr + ROPE_W, ROPE_W, ROPE_H, G_rope_1, FALSE);
}

/*****************************************************/
/*													 */
/*	Draw Player Sprite Hands						 */
/*												     */
/*****************************************************/
void DrawHandRight(u8 yHands)
{
    u8 dir = _dir;
    BOOL flip = (dir & (TOP | RIGHT));

    u8* spriteComp = GetPlayerSpriteHandRight();
    u8 objRight = GetObjectInHand(HAND_RIGHT);

    u8 cyHands;
    u8 xHandRight;
    u8* drawPtr;

    if (IsTorchInHand())
    {
        cyHands = HAND_TORCH_H;
        xHandRight = GetPosxTorch();
    }
    else
    {
        cyHands = HAND_EMPTY_H;
        xHandRight = GetPosxHand();
    }

    drawPtr = GetPlayerSpriteBufferPtr(xHandRight, yHands);
    DrawIntoSpritePlayerCompressed(drawPtr, HAND_CX, cyHands, spriteComp, flip);

    if (objRight == CROWBAR)
    {
        DrawCrowBar(yHands, dir);
    }
    else if (objRight == ROPE)
    {
        DrawRope(yHands, dir);
    }
    else if (gTorchHand.lighted)
    {
        DrawPlayerTorchFlame(yHands, dir);
    }
}

/*****************************************************/
/*													 */
/*	Get Player Sprite Legs							 */
/*												     */
/*****************************************************/
u8* GetPlayerWalkSpriteLegs()
{
    u8 playerStep = _playerStepCycle;
    u8 legAnim = ModuloPowerTwo(_walkStep, 2);

    if (playerStep != STOP_MOVE)
        _walkStep++;
    else
        _walkStep = 0;

    if (_dir == DOWN)
    {	
        if (playerStep == STOP_MOVE)
            return G_front_legs_2;

        return G_front_legs_tileset[legAnim];
    }
    
    if (_dir == TOP)
    {
        if (playerStep == STOP_MOVE)
            return G_front_legs_5;

        return G_front_legs_tileset[legAnim + 3];
    }	

    if (_playerMovement == CHANGE_DIR || playerStep == STOP_MOVE)
        return G_latt_legs_8;

    return G_latt_legs_tileset[_walkStep % (LATT_LEGS_TILES_NB - 1)];
}

/*****************************************************/
/*													 */
/*	Get position X high jump Legs					 */
/*												     */
/*****************************************************/
u8 GetPosxHighJumpLegs()
{
    u8 pos = GetDirToPos();
    return _posLegsHighJump[pos];
}

/*****************************************************/
/*													 */
/*	Get hand Y position     						 */
/*												     */
/*****************************************************/
u8 GetHandYPos()
{
    return IsTorchInHand() ? FLAMES_CY : POS_HAND_LEFT_Y;
}

/*****************************************************/
/*													 */
/*	Draw Player Sprite Legs							 */
/*												     */
/*****************************************************/
void DrawPlayerLegs()
{	
    u8 dir = _dir;
    BOOL flip = (dir == RIGHT);

    u8 x = _posLegs[GetDirToPos()];
    u8 y = POS_LEGS_Y;
    u8 cx = HAS_MOVE_X_AXIS(_dir) ? LATT_LEGS_W : FRONT_LEGS_W;
    u8 cy = LATT_LEGS_H;

    u8* spriteComp;
    u8* sprite;
    u8* drawPtr;

    /** Top legs */
    if (HAS_MOVE_Y_AXIS(dir))
    {		
        drawPtr = GetPlayerSpriteBufferPtr(x, POS_LEGS_Y);
        DrawIntoSpritePlayer(drawPtr, TOP_FRONT_LEGS_0_W, TOP_FRONT_LEGS_0_H, (dir == DOWN) ?  top_front_legs_0 : top_front_legs_1);

        cy = FRONT_LEGS_H;
        y += 3;

        flip = ((_walkStep % 6) > 2);
    }

    drawPtr = GetPlayerSpriteBufferPtr(x, y);
    spriteComp = GetPlayerWalkSpriteLegs();

    sprite = GetDecompressSpriteFlip(spriteComp, cx, cy, flip);
    if (HAS_MOVE_Y_AXIS(dir) && flip)
    {
        if (dir == TOP)
            asm_shiftSpritePixelsLeft(sprite, FRONT_LEGS_SZ);
        else
            asm_shiftSpritePixelsRight(sprite, FRONT_LEGS_SZ);
    }

    DrawIntoSpritePlayer(drawPtr, cx, cy, sprite);
}

/*****************************************************/
/*													 */
/*	Draw Player Front Crouch Legs					 */
/*												     */
/*****************************************************/
void DrawCrouchFrontLegsSprite(u8 indexMove)
{
    u8 x = GetPosxHighJumpLegs();
    u8* drawPtr = GetPlayerSpriteBufferPtr(x, POS_LEGS_Y + (LEGS_CY - FRONT_HIGH_JUMP_H) + 3);
    u8* spriteComp = G_front_high_jump_tileset[indexMove];
    DrawIntoSpritePlayerCompressedMirrored(drawPtr, FRONT_HIGH_JUMP_W, FRONT_HIGH_JUMP_H, (_dir == TOP) ? RIGHT : LEFT, spriteComp);
}

/*****************************************************/
/*													 */
/*	Draw Player Long Jump							 */
/*												     */
/*****************************************************/
void DrawPlayerLongJump()
{
    u8* longJumpSprite;

    BOOL flip = HAS_RIGHT(_dir);

    u8 indexSprite;
    u8 xHead;
    u8 yHead = POS_Y_JUMP_LONG_HEAD;

    u8 playerStep = _playerStepCycle;

    if (playerStep == 0 || playerStep == (NB_STEP_JUMP_LONG - 1))
    {
        indexSprite = 0;
        xHead = flip ? (LONG_JUMP_W - LATT_HEAD_W) - POS_X_JUMP_LONG : POS_X_JUMP_LONG;
        yHead = POS_Y_JUMP_LONG_HEAD + 4;
    }
    else if (playerStep == 1)
    {
        indexSprite = 1;
        xHead = flip ? (LONG_JUMP_W - LATT_HEAD_W) - (POS_X_JUMP_LONG + 1) : POS_X_JUMP_LONG + 1;
    }
    else
    {
        indexSprite = 2;
        xHead = flip ? (LONG_JUMP_W - LATT_HEAD_W) - (POS_X_JUMP_LONG + 3) : POS_X_JUMP_LONG + 3;		
    }

    longJumpSprite = GetDecompressSpriteFlip(G_long_jump_tileset[indexSprite], LONG_JUMP_W, LONG_JUMP_H, flip);   
    DrawIntoSpritePlayer(_longJumpDrawPtr, LONG_JUMP_W, LONG_JUMP_H, longJumpSprite);

    DrawIntoSpritePlayer(GetPlayerSpriteBufferPtr(xHead, yHead), LATT_HEAD_CX, LATT_HEAD_CY, flip ? (latt_head + LATT_HEAD_SZ) : latt_head);
}

/*****************************************************/
/*													 */
/*	Draw Player Front High Jump						 */
/*												     */
/*****************************************************/
void DrawPlayerFrontHighJump()
{
    u8 yChest = POS_CHEST_Y + FRONT_OFFSET;
    u8 yHands = GetHandYPos() + FRONT_OFFSET;

    u8 spriteIndex = HAS_TOP(_dir) ? 3 : 0;
    u8 playerStep = _playerStepCycle;

    if (playerStep == 0 || playerStep == (NB_STEP_JUMP_HIGH - 1))
    {
        yHands += FIRST_FRONT_OFFSET;
        yChest = POS_CHEST_Y + FRONT_OFFSET + FIRST_FRONT_OFFSET;
    }
    else if (playerStep == 1 || playerStep == (NB_STEP_JUMP_HIGH - 2))
    {
        spriteIndex++;
    }
    else
    {
        spriteIndex += 2;
    }

    DrawPlayerChest(yChest);
    DrawHandLeft(yChest);
    DrawHandRight(yHands);
    DrawCrouchFrontLegsSprite(spriteIndex);
}

/*****************************************************/
/*													 */
/*	Draw Player Latteral High Jump Legs				 */
/*												     */
/*****************************************************/
void DrawPlayerLattLegsHighJump(u8 indice)
{
    #define POS_LATT_LEGS_HIGHT_JUMP	(POS_LEGS_Y + (LEGS_CY - LATT_HIGH_JUMP_H) + 3)

    u8 x = GetPosxHighJumpLegs();	
    u8* drawPtr = GetPlayerSpriteBufferPtr(x, POS_LATT_LEGS_HIGHT_JUMP);
    u8* spriteComp = G_latt_high_jump_tileset[indice];

    DrawIntoSpritePlayerCompressed(drawPtr, LATT_HIGH_JUMP_W, LATT_HIGH_JUMP_H, spriteComp, HAS_RIGHT(_dir));
}

/*****************************************************/
/*													 */
/*	Draw Player Latteral High Jump					 */
/*												     */
/*****************************************************/
void DrawPlayerLattHighJump()
{
    u8 yChest = POS_CHEST_Y;
    u8 yHands = GetHandYPos();
    u8 playerStep = _playerStepCycle;
    u8 spriteIndex;

    if (playerStep == 0 || playerStep == (NB_STEP_JUMP_HIGH - 1))
    {
        yHands += 10;
        yChest = POS_CHEST_Y + 10;

        spriteIndex = 0;
    }
    else if (playerStep == 1 || playerStep == (NB_STEP_JUMP_HIGH - 2))
    {
        yHands += 6;
        yChest = POS_CHEST_Y + 6;

        spriteIndex = 1;
    }
    else if (playerStep == 2 || playerStep == (NB_STEP_JUMP_HIGH - 3))
    {
        spriteIndex = 2;
    }
    else
    {
        spriteIndex = 3;
    }

    DrawPlayerChest(yChest);		
    DrawHandRight(yHands);
    DrawHandLeft(yChest);	
    DrawPlayerLattLegsHighJump(spriteIndex);
}

/*****************************************************/
/*													 */
/*	Draw Player	Walk								 */
/*												     */
/*****************************************************/
void DrawPlayerWalk()
{
    DrawPlayerLegs();

    if (_dir != TOP)
        DrawPlayerChest(POS_CHEST_Y);

    DrawHandRight(GetHandYPos());
    DrawHandLeft(POS_CHEST_Y);

    if (_dir == TOP)
        DrawPlayerChest(POS_CHEST_Y);
}

/*****************************************************/
/*													 */
/*	Draw Player Latteral Crouch Legs				 */
/*												     */
/*****************************************************/
void DrawCrouchLattLegsSprite(u8 indexMove, u8 yLegs)
{	
    u8 x = GetPosxHighJumpLegs();
    u8* drawPtr = GetPlayerSpriteBufferPtr(x, yLegs);
    u8* spriteComp = G_latt_high_jump_tileset[indexMove];

    DrawIntoSpritePlayerCompressed(drawPtr, LATT_HIGH_JUMP_W, LATT_HIGH_JUMP_H, spriteComp, HAS_RIGHT(_dir));
}

/*****************************************************/
/*													 */
/*	Draw Player Latteral Crouch						 */
/*												     */
/*****************************************************/
void DrawPlayerLattCrouch()
{
    #define OFFSET_LATT_CROUCH_START	6
    #define OFFSET_LATT_CROUCH_END		10

    u8 yChest;
    u8 yHands = GetHandYPos();
    u8 spriteIndex = 0;

    if (_playerStepCycle == 0)
    {
        yChest = POS_CHEST_Y +  OFFSET_LATT_CROUCH_START;
        yHands += OFFSET_LATT_CROUCH_START;
        spriteIndex = 1;
    }
    else
    {
        yChest = POS_CHEST_Y + OFFSET_LATT_CROUCH_END;
        yHands += OFFSET_LATT_CROUCH_END;
    }

    DrawPlayerChest(yChest);	
    DrawCrouchLattLegsSprite(spriteIndex, POS_LEGS_Y);
    DrawHandRight(yHands);
    DrawObjectInHandLeft(yChest);	
}

/*****************************************************/
/*													 */
/*	Draw Player Front Crouch						 */
/*												     */
/*****************************************************/
void DrawPlayerFrontCrouch()
{
    #define OFFSET_FRONT_CROUCH_START	7
    #define OFFSET_FRONT_CROUCH_END		12

    u8 yChest;
    u8 yHands = GetHandYPos();
    u8 spriteIndex = HAS_TOP(_dir) ? 3 : 0;

    if (_playerStepCycle == 0 || _playerMovement != CROUCH_DOWN)
    {
        yHands += OFFSET_FRONT_CROUCH_START;
        yChest = POS_CHEST_Y + OFFSET_FRONT_CROUCH_START;
        spriteIndex++;
    }
    else 
    {
        yHands += OFFSET_FRONT_CROUCH_END;
        yChest = POS_CHEST_Y + OFFSET_FRONT_CROUCH_END;
    }

    DrawPlayerChest(yChest);		
    DrawHandLeft(yChest);
    DrawHandRight(yHands);
    DrawCrouchFrontLegsSprite(spriteIndex);
}

/*****************************************************/
/*													 */
/*	Draw Player	Sprite								 */
/*												     */
/*****************************************************/
void DrawPlayerToBuffer()
{
    u8 playerMovement = _playerMovement;

    if (HAS_WALKING(playerMovement))
    {
        DrawPlayerWalk();
    }
    else
    if (playerMovement == LONG_JUMP)
    {
        DrawPlayerLongJump();
    }
    else
    if (playerMovement == HIGH_JUMP)
    {
        if (HAS_MOVE_X_AXIS(_dir))
            DrawPlayerLattHighJump();
        else
            DrawPlayerFrontHighJump();
    }
    else
    if (HAS_CROUCHING(playerMovement))
    {
        if (HAS_MOVE_X_AXIS(_dir))
            DrawPlayerLattCrouch();
        else
            DrawPlayerFrontCrouch();
    }
}

/*****************************************************/
/*													 */
/*	Render Player on screen							 */
/*												     */
/*****************************************************/
void RenderPlayer()
{
    if (_playerMovement != LONG_JUMP)
        cpct_waitVSYNC();

    cpct_drawSprite(_spritePlayer, GetScreenPlayer(), PLAYER_SPRITE_CX, PLAYER_SPRITE_CY);
}

/*****************************************************/
/*													 */
/*	Draw stay Player								 */
/*												     */
/*****************************************************/
void RedrawPlayer()
{
    RedrawIntoBuffer();

    if (HasLightInRoom())
        RenderSprite();
}

/*****************************************************/
/*													 */
/*	Draw stay Player								 */
/*												     */
/*****************************************************/
void DrawPlayerStay()
{
    _playerStepCycle = STOP_MOVE;
    RedrawPlayer();
}