//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** CPCTelera includes */
#include <cpctelera.h>
#include "../wincpctelera/wincpctelera_helper.h"

/** Structures includes */
#include "../structures/structures.h"

/** Game includes */
#include "../tools/tools.h"
#include "../video/video.h"
#include "../text/text.h"
#include "../sprites/functions/moveSprite.h"
#include "../input/input.h"
#include "../sound/sound.h"
#include "../ui/uiboard.h"
#include "../text/fontsHelper.h"

/** Room includes */
#include "../room/room.h"
#include "../room/roomDrawItems.h"
#include "../world/world.h"

/** Player include */
#include "player.h"
#include "playerMove.h"
#include "playerDraw.h"

/** Player attributes */
u8 _move;
u8 _playerNbStep;
SCoord _prevPos;

/** Shared player variables */
u8 _playerStepCycle;
u8 _playerMovement;
u8* _currentItemPtrId;
u8 _dir;
u8 _mode;

/** External timer variables */
extern u8 gTimer;

/** Local defines */
#define TOP_LIMIT_Y			5

#define MIN_ROOM_Y			0
#define MAX_ROOM_Y			2
#define MIN_ROOM_X			0
#define MAX_ROOM_X			2

#define LIMIT_HORIZ_LEFT	1
#define LIMIT_HORIZ_RIGHT	64

#define KEYBOARD_TEMPO		20

static const SCoord _enterRoom[4] = {	{ 35, TOP_LIMIT_Y },	// DOOR_TOP
                                        { 64, 65 },				// DOOR_LEFT
                                        { 3,  65 }, 			// DOOR_RIGHT
                                        { 33, 65 } };			// DOOR_DOWN

/** Local functions */
extern void SetRoomPosY();
extern BOOL PlayerExiting();

/*****************************************************/
/*													 */
/*	Initialize Player								 */
/*												     */
/*****************************************************/
void InitPlayer()
{
    _playerStepCycle = 0;
    _playerNbStep = 0;
    _playerMovement = WALK;

    _move = NO_MOVE;
    _dir = DOWN;
    _mode = MODE_MOVE;
}

/*****************************************************/
/*													 */
/*	Save player position							 */
/*												     */
/*****************************************************/
void SavePlayerPos()
{
    if (gRoomId != PIT_ROOM)
    {
        _prevPos.x = gPlayerPos.x;
        _prevPos.y = gPlayerPos.y;
    }
}

/*****************************************************/
/*													 */
/*	Restore player position							 */
/*												     */
/*****************************************************/
void RestorePlayerPos()
{
    gPlayerPos.x = _prevPos.x;
    gPlayerPos.y = _prevPos.y;
    SetRoomPosY();
}

/*****************************************************/
/*													 */
/*	Restore falling in trap							 */
/*												     */
/*****************************************************/
void IsFallingInTrap(u8 playerMove)
{
    if (*_currentItemPtrId == ROOM_ITEM_HIDE_TRAP || (*_currentItemPtrId == ROOM_ITEM_TRAP && playerMove == LONG_JUMP))
    {
        SetPlayerDir(TOP);
        ExitRoom(FALL_PIT);

        *_currentItemPtrId = ROOM_ITEM_TRAP;
        _playerNbStep = 0;
    }
}

/*****************************************************/
/*													 */
/*	Get player dir									 */
/*												     */
/*****************************************************/
u8 GetPlayerDir()
{
    return _dir;
}

/*****************************************************/
/*													 */
/*	End player move									 */
/*												     */
/*****************************************************/
void EndPlayerMove()
{
    u8 dir = _dir;

    if (HAS_JUMPING(_playerMovement))
        PlayFootStep();

    _playerMovement = WALK;

    if (HAS_MOVE_X_AXIS(dir))
        _dir = HAS_RIGHT(dir) ? RIGHT : LEFT;
    else
        _dir = HAS_DOWN(dir) ? DOWN : TOP;

    _playerStepCycle = STOP_MOVE;
}

/*****************************************************/
/*													 */
/*	Get limit of room in direction					 */
/*												     */
/*****************************************************/
u8 GetLimitOfRoom(u8 dir)
{
    #define LIMIT_LEFT_PIT		26
    #define LIMIT_RIGHT_PIT		41
    #define LIMIT_LAC			26

    u8 limit;

    if (gRoomId == LAC_FOREST_ROOM && dir == LIMIT_LEFT)
    {
        limit = LIMIT_LAC;
    }
    else if (gRoomSize == ROOM_PIT)
    {
        limit = (dir == LIMIT_LEFT) ? LIMIT_LEFT_PIT : LIMIT_RIGHT_PIT;
    }
    else
    {
        u8 roomY = gPlayerRoomY;
        limit = (dir == LIMIT_LEFT) ?
            !HasCurrentRoomWall(DOOR_LEFT) ? LIMIT_HORIZ_LEFT : GetLimitHoriz(roomY, LIMIT_LEFT) :
            !HasCurrentRoomWall(DOOR_RIGHT) ? LIMIT_HORIZ_RIGHT : GetLimitHoriz(roomY, LIMIT_RIGHT);
    }

    return limit;
}

/*****************************************************/
/*													 */
/*	Is player in front of top wall room				 */
/*												     */
/*****************************************************/
BOOL IsFrontTopWall()
{
    return (_dir == TOP && gPlayerRoomY == (3 - gRoomSize) && HasCurrentRoomWall(DOOR_TOP));
}

/*****************************************************/
/*													 */
/*	If item can be walk on							 */
/*												     */
/*****************************************************/
BOOL IsDecorItem(u8 itemId)
{
    return (itemId == ROOM_EVENT_LIGHT_OFF || itemId == ROOM_SKELETON) || (itemId == ROOM_DEMON_CIRCLE && (gAdventure & ADV_DEMON_VANISHED));
}

/*****************************************************/
/*													 */
/*	Event when walk on it					 		 */
/*												     */
/*****************************************************/
void EventAction()
{
    if (*_currentItemPtrId == ROOM_EVENT_LIGHT_OFF && gTorchHand.lighted && !IsRoomLit())
        ExtinguishTorch();
}

/*****************************************************/
/*													 */
/*	If Player can move								 */
/*												     */
/*****************************************************/
u8 CanMove(u8 move)
{
    u8 y = gPlayerPos.y;
    u8 roomY = gPlayerRoomY;
    u8 maxRoomY = (gRoomSize == ROOM_LARGE) ? ROOM_LARGE : 1;

    int top = (ROOM_MAX_SIZE - maxRoomY) * NB_STEP_Y * 2 + TOP_LIMIT_Y;
    int down = MAX_ROOM_Y * NB_STEP_Y * 2 + TOP_LIMIT_Y;

    if (move == TOP && y <= top)
    {
        y = top;
        return COLLIDE_ROOM;
    }
    else if (move == DOWN && y >= down)
    {
        y = down;
        return COLLIDE_ROOM;
    }
    else
    {
        u8 x = gPlayerPos.x;		
        _currentItemPtrId = NULL;

        if ( HAS_MOVE_Y_AXIS(move) ||
            (HAS_LEFT(move) && x > GetLimitOfRoom(LIMIT_LEFT)) ||
            (HAS_RIGHT(move) && x < GetLimitOfRoom(LIMIT_RIGHT)))
        {
            u8 itemId = NOTHING;
            u8* itemPtr = GetCollideGroundItem();
            if (itemPtr != NULL)
            {
                _currentItemPtrId = itemPtr;
                itemId = *itemPtr;

                EventAction();
            }

            return (itemId < OBJECTS || IsDecorItem(itemId)) ? NO_COLLIDE : COLLIDE_ITEM;
        }
        else
            return COLLIDE_ROOM;
    }
}

/*****************************************************/
/*													 */
/*	Convert direction to position					 */
/*												     */
/*****************************************************/
u8 ConverDirToPos(u8 dir)
{
    static const u8 sPosDoor[] = { DOOR_TOP, DOOR_DOWN, DOOR_LEFT, NOTHING, DOOR_RIGHT };
    return sPosDoor[dir/2];
}

/*****************************************************/
/*													 */
/*	Compute Room position Y							 */
/*												     */
/*****************************************************/
void SetRoomPosY()
{
    gPlayerRoomY = ((gPlayerPos.y + 1) - TOP_LIMIT_Y) / (NB_STEP_Y * 2);
}

/*****************************************************/
/*													 */
/*	Walk Depth room									 */
/*												     */
/*****************************************************/
void WalkDepth(u8 move)
{
    if (move == TOP)
    {
        u8 x = gPlayerPos.x;
        u8 y = gPlayerPos.y;

        BOOL isLimitLeft = (x <= GET_LEFT_LIMIT(y));
        BOOL isLimitRight = (x >= GET_RIGHT_LIMIT(y));

        if (HasCurrentRoomWall(DOOR_RIGHT) && isLimitRight)
        {
            MoveLeftSprite(1);
        }
        else
        if (HasCurrentRoomWall(DOOR_LEFT) && isLimitLeft)
        {
             MoveRightSprite(1);
        }

        MoveTopSprite(2);
    }
    else
    {
        MoveDownSprite(2);
    }
}

/*****************************************************/
/*													 */
/*	Walk Latteral room								 */
/*												     */
/*****************************************************/
void WalkLatteral(u8 move)
{
    (move == RIGHT) ? MoveRightSprite(1) : MoveLeftSprite(1);
}

/*****************************************************/
/*													 */
/*	Walk Player										 */
/*												     */
/*****************************************************/
void Walk(u8 move)
{	
    if (CanMove(move) == NO_COLLIDE)
    {
        switch (move)
        {
            case TOP:	WalkDepth(TOP);			break;
            case DOWN:	WalkDepth(DOWN);		break;
            case RIGHT:	WalkLatteral(RIGHT);	break;
            case LEFT:	WalkLatteral(LEFT);		break;
        }
    }
    else
    {
        if (_currentItemPtrId != NULL)
        {
            IsFallingInTrap(WALK);
        }
        else
        {
            if (!PlayerExiting())
                DrawPlayerStay();
        }
    }
}

/*****************************************************/
/*													 */
/*	Step jump cycle									 */
/*												     */
/*****************************************************/
void StepJump()
{
    if (_playerNbStep == 0)
    {
        EndPlayerMove();
        DrawPlayerStay();
    }
    else
        _playerStepCycle++;
}

/*****************************************************/
/*													 */
/*	Long Jump Player								 */
/*												     */
/*****************************************************/
void LongJump()
{
    if (_playerStepCycle < (NB_STEP_JUMP_LONG / 2))
        MoveTopSprite(2);
    else
        MoveDownSprite(2);

    if (HAS_RIGHT(_dir))
        MoveRightSprite(2);
    else
        MoveLeftSprite(2);

    StepJump();
}

/*****************************************************/
/*													 */
/*	High Jump Player								 */
/*												     */
/*****************************************************/
void HighJump()
{
    #define TORCH_LIT_OFFSET    4

    u8 step = _playerStepCycle;
    if (step > 1)
    {
        if (step <= (NB_STEP_JUMP_HIGH / 2))
            MoveTopSprite(2);
        else
            MoveDownSprite(2);

        if (step == (NB_STEP_JUMP_HIGH / 2) && gPlayerRoomY == MAX_ROOM_Y && IsTorchInHand())
        {
            u8 x = gPlayerPos.x;
            u8 dir = _dir;
            BOOL isNearTorchWall =  (HAS_LEFT(dir) && (x <= GetLimitHoriz(MAX_ROOM_Y, LIMIT_LEFT) + TORCH_LIT_OFFSET) ||
                                    (HAS_RIGHT(dir) && (x >= GetLimitHoriz(MAX_ROOM_Y, LIMIT_RIGHT) - TORCH_LIT_OFFSET)));

            if (isNearTorchWall)
                LightRoomTorch(dir);
        }

        if ((step == NB_STEP_JUMP_HIGH - 4) && IsRoomLit())
            DrawTorch();        
    }
    else
    {
        RedrawPlayer();
    }

    StepJump();
}

/*****************************************************/
/*													 */
/*	Crouch Player									 */
/*												     */
/*****************************************************/
void Crouch()
{
    if (_playerStepCycle == 1 && _playerMovement == CROUCH_UP)
        EndPlayerMove();

    RedrawPlayer();

    if (_playerNbStep != 0)
        _playerStepCycle++;
}

/*****************************************************/
/*													 */
/*	Change direction								 */
/*												     */
/*****************************************************/
void ChangeDir(u8 move)
{
    u8 dir = _dir;
    _dir = move;

    if (move == DOWN && dir == TOP)
    {
        _dir = LEFT;
    }
    else
    if (move == TOP && dir == DOWN)
    {
        _dir = RIGHT;
    }
    else
    if (move == LEFT && dir == RIGHT)
    {
        _dir = TOP;
    }
    else
    if (move == RIGHT && dir == LEFT)
    {
        _dir = DOWN;
    }

    DrawPlayerStay();
}

/*****************************************************/
/*													 */
/*	Animate Player									 */
/*												     */
/*****************************************************/
void AnimatePlayer(u8 move)
{
    if (_playerNbStep > 0)
    {
        u8 playerMove = _playerMovement;
        _playerNbStep--;

        switch (playerMove)
        {
            case WALK			: Walk(move);			break;
            case CHANGE_DIR		: ChangeDir(move); 		break;
            case LONG_JUMP		: LongJump();			break;
            case HIGH_JUMP		: HighJump();			break;
            case CROUCH_DOWN	: 
            case CROUCH_UP		: Crouch();				break;
        }	

        if (_playerNbStep == 0)
        {
            if (playerMove == WALK)
            {
                if (HAS_MOVE_Y_AXIS(move))
                    SetRoomPosY();
            }
            else	
            if (playerMove == LONG_JUMP)
            {
                u8* itemPtr = GetCollideGroundItem();
                if (itemPtr != NULL)
                {
                    _currentItemPtrId = itemPtr;
                    IsFallingInTrap(playerMove);
                }
            }
        }
    }
}

/*****************************************************/
/*													 */
/*	Is Player can exit by direction					 */
/*												     */
/*****************************************************/
u8 CanExitDir(u8 dir)
{
    #define DOOR_EXIT_LEFT	28
    #define DOOR_EXIT_RIGHT	40

    if (_dir == dir)
    {
        u8 x = gPlayerPos.x;
        u8 y = gPlayerPos.y;
        u8 roomMinY = (gRoomSize == ROOM_LARGE) ? MIN_ROOM_Y : MAX_ROOM_Y;
        u8 roomY = gPlayerRoomY;
        u8 playerMovement = _playerMovement;

        u8 doorPos = ConverDirToPos(dir);
        u8 door = gCurrentRoom->doorDef[doorPos];
        BOOL hasWall = (door != DOOR_TYPE_NO_WALL);

        if (HAS_MOVE_X_AXIS(dir) && (playerMovement & (WALK|CHANGE_DIR)))
        {
            if ((dir == LEFT && x <= GET_LEFT_LIMIT(y)) || (dir == RIGHT && x >= GET_RIGHT_LIMIT(y)))
            {
                // Test if open wall exit
                if (!hasWall)
                    return EXIT_WALL;

                // Test if not in front of door
                if (roomY != MAX_ROOM_Y)
                    return NO_MOVE;

                // Test if door opened
                return IsDoorOpened(doorPos) ? EXIT_DOOR : EXIT_DOOR_CLOSED;
            }
        }
        else
        {
            // Test if open wall exit or in front of Door
            if ((dir == TOP && roomY == roomMinY) || (dir == DOWN && roomY == MAX_ROOM_Y))
            {
                // Test if open wall exit
                if (!hasWall)
                    return EXIT_WALL;

                // Test if not in front of door
                if (x < DOOR_EXIT_LEFT || x > DOOR_EXIT_RIGHT)
                    return NO_MOVE;

                // Test if door opened
                if (IsDoorOpened(doorPos))
                {
                    if (GET_DOOR_TYPE(door) == DOOR_TYPE_SECRET)
                    {
                        if (playerMovement != CROUCH_DOWN)
                            PrintTextId(TEXT_MUST_CROUCH);
                        else
                            return EXIT_DOOR;
                    }
                    else if (playerMovement != CROUCH_DOWN)
                    {
                        return EXIT_DOOR;
                    }
                }
                return EXIT_DOOR_CLOSED;
            }
        }
    }
    // Change direction
    return NO_MOVE;
}

/*****************************************************/
/*													 */
/*	Player can exit room							 */
/*												     */
/*****************************************************/
BOOL PlayerExiting()
{
    u8 exit = CanExitDir(_dir);
    if (exit != EXIT_DOOR_CLOSED && exit != NO_MOVE)
    {
        _playerNbStep = 0;
        ExitRoom(_dir);
        return TRUE;
    }
    return FALSE;
}

/*****************************************************/
/*													 */
/*	Can Player Long Jump							 */
/*												     */
/*****************************************************/
BOOL CanLongJump()
{
    #define DIST_LONG_JUMP	(NB_STEP_JUMP_LONG * 2)
    
    u8 x = gPlayerPos.x;
    BOOL canJump = ((gRoomId != DEMON_ROOM) &&
                    (_dir == LEFT && ((x - DIST_LONG_JUMP) >= GetLimitOfRoom(LIMIT_LEFT)) ||
                    (_dir == RIGHT && ((x + DIST_LONG_JUMP) <= GetLimitOfRoom(LIMIT_RIGHT)))));
                   
    return canJump;
}

/*****************************************************/
/*													 */
/*	Player Walk										 */
/*												     */
/*****************************************************/
void PlayerWalk(u8 move)
{
    u8 collide = CanMove(move);
    if (collide == NO_COLLIDE)
    {
        _playerMovement = WALK;
        _playerNbStep = HAS_MOVE_Y_AXIS(move) ? NB_STEP_Y : NB_STEP_X;		

        SavePlayerPos();
    }
    else if ((collide == COLLIDE_ROOM && !PlayerExiting()) || collide == COLLIDE_ITEM)
    {
        _playerStepCycle = 0;
        DrawPlayerStay();

        if (collide == COLLIDE_ITEM)
            IsFallingInTrap(WALK);
    }
}

/*****************************************************/
/*													 */
/*	Move Player										 */
/*												     */
/*****************************************************/
void SetMovePlayer(u8 move)
{
    if (move != NO_MOVE)
    {
        BOOL sameDir = (GET_DIR(move) == GET_DIR(_dir));
        _playerStepCycle = 0;

        if (_playerMovement == CROUCH_DOWN)
        {
            if (!PlayerExiting() && HAS_TOP(move))
            {
                _playerMovement = CROUCH_UP;
                _playerNbStep = NB_STEP_CROUCH;
            }
        }
        else if (IS_FIRE_LAT(move) && sameDir && gPlayerRoomY != 0 && CanLongJump())
        {
            SavePlayerPos();

            _playerMovement = LONG_JUMP;
            _playerNbStep = NB_STEP_JUMP_LONG;
            _dir = move;
        }
        else if (IS_FIRE_TOP(move) && gPlayerRoomY != 0)
        {
            _playerMovement = HIGH_JUMP;
            _playerNbStep = NB_STEP_JUMP_HIGH;
        }
        else if (IS_FIRE_DOWN(move))
        {
            _playerMovement = CROUCH_DOWN;
            _playerNbStep = NB_STEP_CROUCH;
        }
        else if (!HAS_FIRE(move))
        {
            /** Walk */
            if (sameDir)
            {
                PlayerWalk(move);
            }
            /** Change direction */
            else
            {
                _playerMovement = CHANGE_DIR;
                _playerNbStep = WAIT_CHANGE_DIR;
            }
        }
    }
}

/*****************************************************/
/*													 */
/*	Set Player Direction							 */
/*												     */
/*****************************************************/
void SetPlayerDir(u8 dir)
{
    _dir = dir;
}

/*****************************************************/
/*													 */
/*	Set Player Position when enter room				 */
/*												     */
/*****************************************************/
void SetPlayerPosition(u8 door)
{
    u8 dir = _dir;
    u8 doorType = GET_DOOR_TYPE(door);

    if (HAS_MOVE_Y_AXIS(dir))
    {		
        // X position
        if (HaveDoor(door))
        {
            gPlayerPos.x = _enterRoom[dir == TOP ? DOOR_TOP : DOOR_DOWN].x;
        }
        else if (dir == DOWN)
        {
            u8 limitLeft = GetLimitHoriz(0, LIMIT_LEFT);			
            u8 x = gPlayerPos.x;

            if (x < limitLeft)
            {
                gPlayerPos.x = limitLeft;
            }
            else
            {
                u8 limitRight = GetLimitHoriz(0, LIMIT_RIGHT);
                if (x > limitRight)
                    gPlayerPos.x = limitRight;
            }
        }
        else
            gPlayerPos.x = ROOM_CX / 2;

        // Y position
        if ((dir == TOP || gRoomSize == ROOM_CORRIDOR) && doorType != DOOR_TYPE_SECRET)
        {
            gPlayerPos.y = _enterRoom[DOOR_DOWN].y;
            gPlayerRoomY = MAX_ROOM_Y;			
        }
        else
        {
            gPlayerPos.y = _enterRoom[DOOR_TOP].y;
            gPlayerRoomY = MIN_ROOM_Y;
        }

        if (doorType == DOOR_TYPE_SECRET)
            _dir = DOWN;
    }
    else if (dir == LEFT)
    {
        gPlayerPos.x = HasCurrentRoomWall(DOOR_RIGHT) ? _enterRoom[DOOR_LEFT].x : LIMIT_HORIZ_RIGHT;	
    }
    else
    {
        gPlayerPos.x = HasCurrentRoomWall(DOOR_LEFT) ?  _enterRoom[DOOR_RIGHT].x : LIMIT_HORIZ_LEFT;	
    }

    if (IS_WOOD_DOOR_TYPE(doorType))
        PlayDoor();
}

/*****************************************************/
/*													 */
/*	Get current player move							 */
/*												     */
/*****************************************************/
u8 GetPlayerMovement()
{
    return _playerMovement;
}

/*****************************************************/
/*													 */
/*	Check player move								 */
/*												     */
/*****************************************************/
void UpdatePlayerMove()
{
    if (_playerNbStep == NO_MOVE)
    {
        u8 move = _move = CheckUserInput();		
        if (move != NO_MOVE)
        {
            if (move == SUICIDE && _mode == MODE_MOVE)
            {
                PrintTextId(TEXT_SUICIDE);

                WaitForKeyReleased(SUICIDE);
                while (CheckUserInput() == NO_MOVE);

                if (CheckUserInput() == SUICIDE)
                    PlayerDead();
                else
                    DisplayRoomText();
            }
            else
            {
                if (_mode == MODE_MOVE)
                {
                    if (move == SELECT)
                    {
                        _mode = MODE_INV;
                        SetSelection(NO_MOVE);
                    }
                    else
                    {
                        if (!IS_FIRE(move))
                            SetMovePlayer(move);
                        else
                            InteractPlayer(NOTHING);
                    }
                }
                else
                {
                    if (!SetSelection(move))
                        _mode = MODE_MOVE;
                }
            }
        }
    }
    
    if (_playerNbStep != NO_MOVE)
        AnimatePlayer(_move);
    else
        DrawPlayerStay();
                
    WinCpcTelera_Wait(100);

   /* WinCpcTelera_PrintInt("X", gPlayerPos.x);
    WinCpcTelera_PrintInt("Y", gPlayerPos.y);
    WinCpcTelera_PrintInt("ROOM X", gPlayerRoom.x);
    WinCpcTelera_PrintInt("ROOM Y", gPlayerRoomY);
    WinCpcTelera_PrintInt("ROOM", gRoomId);
    WinCpcTelera_PrintInt("STEPS", _playerStepCycle);*/
}
