//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** CPCTelera includes */
#include <cpctelera.h>
#include "../wincpctelera/wincpctelera_helper.h"

/** Structures includes */
#include "../structures/structures.h"

/** Decors Sprites */
#include "../sprites/allSprites.h"

/** Room include */
#include "room.h"
#include "roomDrawCastle.h"
#include "roomDrawForest.h"
#include "roomDrawItems.h"

/** Player includes */
#include "../player/player.h"
#include "../player/playerDraw.h"
#include "../player/playerMove.h"

/** Game includes */
#include "../input/input.h"
#include "../sound/sound.h"
#include "../tools/tools.h"
#include "../items/torchFlame.h"
#include "../monsters/demon.h"
#include "../world/world.h"
#include "../text/text.h"

/** Video includes */
#include "../video/video.h"
#include "../sprites/functions/packedSprites.h"

/** Ui includes */
#include "../ui/uiBoard.h"
#include "../ui/uiShop.h"

/** Text includes */
#include "../text/fontsHelper.h"

/** Room defines */
#define STONE_NUMBER_HEIGHT		2

/** Local variables */
static const u8 _limitHoriz[3][3] = {	{ 18, 49, 49 - 18 },
                                        { 11, 57, 57 - 11 },
                                        { 4,  64, 64 - 4 } 	};

/** Room variables */
BOOL _leftWall;
BOOL _rightWall;
BOOL _topWall;
BOOL _downWall;

u8 _cyStones;
u8 _posLeftAngle;
u8 _posRightAngle;
u8 _posFrontWall;
u8 _startGround;
u8* _doorDest;

u8 gRoomSize;
u8 gRoomType;
u8 gRoomId;
u8 gPrevRoomId;
SRoom* gCurrentRoom;
u8* gWallLatt;

/** Extern variables */
extern volatile BOOL gDrawRoomInProgress;
extern u8 gTempoMonster;
extern u8 gColorSky;
extern SWorldStatic* gWorldStatic;

/*****************************************************/
/*												     */
/*	Update current Room								 */
/*												     */
/*****************************************************/
void UpdateCurrentRoom(u8 roomId)
{
    SRoomStatic* roomStatic = &gWorldStatic->roomsStatic[roomId];
    u8 sizeRoomId = roomStatic->size_roomId;
    gRoomSize = GET_ROOM_SIZE(sizeRoomId);
    gRoomId = GET_ROOM_ID(sizeRoomId);

    _doorDest = roomStatic->doorDest;
    gWallLatt = roomStatic->wallLatt;

    gCurrentRoom = GetRoom(roomId);

    InitRoomVar();
}

/*****************************************************/
/*												     */
/*	Init Rooms										 */
/*												     */
/*****************************************************/
void InitRooms()
{
    gRoomId = ROOM_START;
    UpdateCurrentRoom(ROOM_START);
}

/*****************************************************/
/*												     */
/*	Get Room to										 */
/*												     */
/*****************************************************/
u8 GetNextRoomIdTo(u8 pos)
{
    return _doorDest[pos];
}

/*****************************************************/
/*													 */
/*	Has current Room Wall							 */
/*												     */
/*****************************************************/
BOOL HasCurrentRoomWall(u8 pos)
{
    return (gCurrentRoom->doorDef[pos]!= DOOR_TYPE_NO_WALL);
}

/*****************************************************/
/*													 */
/*	Has Room Wall									 */
/*												     */
/*****************************************************/
BOOL HasRoomWall(u8 roomId, u8 pos) 
{
    SRoom* room = GetRoom(roomId);
    return (room->doorDef[pos] != DOOR_TYPE_NO_WALL);
}

/*****************************************************/
/*												     */
/*	Init room										 */
/*												     */
/*****************************************************/
void InitRoomVar()
{
    // CORRIDOR - MEDIUM - LARGE
    static const u8 sStonesRoom[] = { 5 * STONE_NUMBER_HEIGHT, 4 * STONE_NUMBER_HEIGHT, 3 * STONE_NUMBER_HEIGHT };
    static const u8 sPosAngle[] = { POS_STONE_ANGLE_X - 13, POS_STONE_ANGLE_X - 7, POS_STONE_ANGLE_X };
    static const u8 sFrontWall[] = { POS_WALL_FRONT_Y + 38, POS_WALL_FRONT_Y, POS_WALL_FRONT_Y };

    _leftWall = HasCurrentRoomWall(DOOR_LEFT);
    _rightWall = HasCurrentRoomWall(DOOR_RIGHT);
    _topWall = HasCurrentRoomWall(DOOR_TOP);
    _downWall = HasCurrentRoomWall(DOOR_DOWN);

    gRoomType = GET_ROOM_TYPE(gCurrentRoom->type_light_torch);

    _cyStones = 2;
    if (_topWall)
    {
        u8 roomSize = gRoomSize - 1;
        _cyStones = sStonesRoom[roomSize];
        _posLeftAngle = sPosAngle[roomSize];
        _posRightAngle = (ROOM_CX - _posLeftAngle - STONE_ANGLE_W);
        _posFrontWall = sFrontWall[roomSize];
    }

    _startGround = _cyStones * STONE_ANGLE_OFFSET - POS_STONE_ANGLE_Y + 1;
    if (gRoomSize == ROOM_CORRIDOR)
        _startGround += 5;

    gTempoMonster = 0;

    SetRandSeed(gRoomId);
}

/*****************************************************/
/*													 */
/*	Has light in room								 */
/*												     */
/*****************************************************/
BOOL IsRoomLit()
{
    u8 roomTorch = gCurrentRoom->type_light_torch;
    return  (HAS_TORCH_BURNISH(roomTorch) || (HAS_ROOM_DAYLIGHT(roomTorch) && !IsGameWin()));
}

/*****************************************************/
/*													 */
/*	Has light in room or torch hand					 */
/*												     */
/*****************************************************/
BOOL HasLightInRoom()
{
    return (gRoomId == WIN_ROOM || gTorchHand.lighted || IsRoomLit());
}

/*****************************************************/
/*													 */
/*	Set Room darkness								 */
/*												     */
/*****************************************************/
void SetRoomDarkness()
{
    if (!HasLightInRoom())
    {
        gDrawRoomInProgress = TRUE;
        CleanRoom(ROOM_CY);
        gDrawRoomInProgress = FALSE;
        
        SetupPlayerSpriteMove();
        DisplayRoomText();
    }
}

/*****************************************************/
/*													 */
/*	Display text info room							 */
/*												     */
/*****************************************************/
void DisplayRoomText()
{
    u8 textId = TEXT_GROUND_FLOOR;

    if (!HasLightInRoom())
    {
        textId = TEXT_ITS_DARK_HERE;
    }
    else
    {
        u8 roomId = gRoomId;

        if (gRoomType == ROOM_FOREST)
            textId = TEXT_OUTDOOR;

        if (roomId == LAC_FOREST_ROOM)
            textId = TEXT_LAC;

        if (roomId >= 20 && roomId <= 28)
            textId = TEXT_BASEMENT;

        if (roomId == 29 || (roomId >= 39 && roomId <= 54))
            textId = TEXT_FRST_FLOOR;

        if ((roomId >= 30 && roomId <= 36) || (roomId >= 56 && roomId <= 62))
            textId = TEXT_LAST_FLOOR;

        if (roomId == CELL_KING_ROOM)
            textId = TEXT_KING_CELL;

        if (roomId == CHAPEL_ROOM)
            textId = TEXT_CHAPEL;

        if (roomId == WIN_ROOM)
            textId = TEXT_WIN;
    }

    PrintTextId(textId);
}

/*****************************************************/
/*													 */
/*	Draw Room										 */
/*												     */
/*****************************************************/
void DrawRoom()
{
    gDrawRoomInProgress = TRUE;

    InitDrawRoom();

    if (gRoomType == ROOM_FOREST)
    {
        DrawRoomForest();
    }
    else
    {
        DrawRoomCastle();

        if (gRoomId == DEMON_ROOM && !(gAdventure & ADV_DEMON_VANISHED))
            DrawDemon();
    }

    gDrawRoomInProgress = FALSE;

    SetupPlayerSpriteMove();
}

/*****************************************************/
/*													 */
/*	Enter Room										 */
/*												     */
/*****************************************************/
void EnterRoom()
{
    DisplayRoomText();
    DrawRoom();

    WinCpcTelera_PrintInt("ROOM ID", gRoomId);
}

/*****************************************************/
/*													 */
/*	Exit Room										 */
/*												     */
/*****************************************************/
void ExitRoom(u8 dir)
{
    u8 prevRoomId = gRoomId;
    
    if (dir >= FALL_PIT && dir <= CLIMB_DOWN_PIT)
    {
        if (dir == CLIMB_UP_PIT)
        {
            UpdateCurrentRoom(gPrevRoomId);
            RestorePlayerPos();
        }
        else	
        {	
            if (dir == FALL_PIT)
                UpdateLife(FALL_LOSE_LIFE);					

            UpdateCurrentRoom((gRoomId == DUNGEON_ROOM) ? DUNGEON_PIT_ROOM : PIT_ROOM);

            PlayFootStep();
            SetPlayerDir(TOP);
            SetPlayerPosition(DOOR_TYPE_WALL);	
        }
    
        SetPlayerDir(DOWN);
    }
    else
    {
        u8 pos = ConverDirToPos(dir);
        u8 toRoomId = _doorDest[pos];
        u8 door = gCurrentRoom->doorDef[pos];

        if (toRoomId == TO_CASTLE_ROOM && IsGameWin())
        {
            toRoomId = WIN_ROOM;
            gColorSky = SKY_COLOR;
            SetPlayerDir(TOP);

            EmptyHand(HAND_RIGHT);
            EmptyHand(HAND_LEFT);
            gTorchHand.lighted = FALSE;
        }
        else
        if (toRoomId == SHOP_ROOM)
        {
            gRoomId = SHOP_ROOM;
            SetPlayerDir(LEFT);
            DrawShop();
            toRoomId = gRoomId = ROOM_START;
        }

        UpdateCurrentRoom(toRoomId);
        SetPlayerPosition(door);
    }

    gPrevRoomId = prevRoomId;
    EnterRoom();
}

/*****************************************************/
/*													 */
/*	Get opposite door								 */
/*												     */
/*****************************************************/
u8 GetOppositeDoor(u8 doorPos)
{
    static const u8 sOppDoor[] = { DOOR_DOWN, DOOR_RIGHT, DOOR_LEFT, DOOR_TOP };
    return sOppDoor[doorPos];
}

/*****************************************************/
/*													 */
/*	Get CrowBar force			    				 */
/*												     */
/*****************************************************/
u8 GetCrowBarForce()
{
    return (GetRandMax(5) > 2) ? 1 : 0;
}

/*****************************************************/
/*													 */
/*	Force chest										 */
/*												     */
/*****************************************************/
BOOL ForceChest()
{
    BOOL opened;
    SChest* chest = GetRoomChest();    
    chest->solidity -= GetCrowBarForce();

    opened = (chest->solidity == 0);
    UseCrowBar(opened);
    return opened;
}

/*****************************************************/
/*													 */
/*	Is chest is opened								 */
/*												     */
/*****************************************************/
BOOL IsChestOpened()
{
    SChest* chest = GetRoomChest();
    return (chest->solidity == 0);
}

/*****************************************************/
/*													 */
/*	Is chest empty									 */
/*												     */
/*****************************************************/
BOOL IsChestEmpty()
{
    u8* items = GetRoomChest()->items;
    return !(items[0] | items[1] | items[2]);
}

extern SWorldStatic* gWorldStatic;

/*****************************************************/
/*													 */
/*	Open door										 */
/*												     */
/*****************************************************/
void OpenDoor(u8* door, u8 doorPos, u8 oppDoor)
{
    u8 nextRoomId = GetNextRoomIdTo(doorPos);
    SRoom* nextRoom = GetRoom(nextRoomId);
    nextRoom->doorDef[oppDoor] |= DOOR_OPENED;
    *door |= DOOR_OPENED;
}

/*****************************************************/
/*													 */
/*	Open secret passage								 */
/*												     */
/*****************************************************/
void TryOpenSecretPassage()
{
    if (gWallLatt[DECO_LAT_LEFT] == WALL_SECRET_KEY_HOLE && gPlayerRoomY == 1 && GetPlayerDir() == LEFT && gPlayerPos.x <= GET_LEFT_LIMIT(gPlayerPos.y))
    {
        u8* door = &gCurrentRoom->doorDef[DOOR_TOP];
        if (!IS_DOOR_OPENED(*door))
        {
            PlayImpact();
            AnimationOpenSecretDoor();	
            OpenDoor(door, DOOR_TOP, DOOR_TOP);
        }
    }
}

/*****************************************************/
/*													 */
/*	Open locked door								 */
/*												     */
/*****************************************************/
void TryOpenLockedDoor(u8 objectId)
{
    u8 dir = GetPlayerDir();
    u8 doorPos = ConverDirToPos(dir);
    if (HasCurrentRoomWall(doorPos))
    {
        u8* doorPtr = &gCurrentRoom->doorDef[doorPos];
        u8 door = *doorPtr;
        u8 doorType = GET_DOOR_TYPE(door);
        if (IS_WOOD_DOOR_TYPE(doorType))
        {
            BOOL opened = FALSE;
            if (!IS_DOOR_OPENED(door))
            {
                u8 lock = GET_LOCK_TYPE(door);
                if (objectId == METAL_KEY)
                {
                    if (lock == DOOR_LOCK_METAL)
                    {
                        PlayFootStep();
                        opened = TRUE;
                    }
                    else
                        PrintTextId(TEXT_WRONG_KEY);
                }
                else if (objectId == CROWBAR)
                {
                    u8 solidity = GET_DOOR_SOLIDITY(door) - GetCrowBarForce();
                    if (solidity != 0)
                        *doorPtr = SET_DOOR_SOLIDITY(door, solidity);
                    else
                        opened = TRUE;

                    UseCrowBar(opened);
                }

                if (opened)
                {
                    u8 oppDoor = GetOppositeDoor(doorPos);
                    OpenDoor(doorPtr, doorPos, oppDoor);
                    PrintTextId(TEXT_UNLOCKED);
                }
            }
        }
    }
}

/*****************************************************/
/*													 */
/*	Is Room door opened								 */
/*												     */
/*****************************************************/
BOOL IsDoorOpened(u8 pos)
{
    u8 door = gCurrentRoom->doorDef[pos];
    u8 doorType = GET_DOOR_TYPE(door);

    /** NO WALL */
    if (doorType == DOOR_TYPE_NO_WALL || doorType == DOOR_TYPE_PASSAGE)
        return TRUE;

    /** DOOR */
    if (doorType != DOOR_TYPE_WALL)
    {
        BOOL doorOpened = IS_DOOR_OPENED(door);
        if (!doorOpened && doorType != DOOR_TYPE_SECRET)
            PrintTextId(TEXT_LOCKED);

        return doorOpened;
    }

    /** WALL */
    return FALSE;
}

/*****************************************************/
/*													 */
/*	Update Room										 */
/*												     */
/*****************************************************/
void UpdateRoom()
{
    u8 roomId = gRoomId;

    if (gRoomType == ROOM_FOREST)
    {
        if (roomId == LAC_FOREST_ROOM)
            DrawWaterReflection();
    }
    else
    {
        if (HasLightInRoom())
        {
            DrawTorchFlames();

            if (roomId == DEMON_ROOM && !(gAdventure & ADV_DEMON_VANISHED))
                DrawDemon();
            else
            if (roomId == DEMON_ROOM_PAINT_1 || roomId == DEMON_ROOM_PAINT_2)
                DrawDemonPaint();
        }
    }
}

/*****************************************************/
/*													 */
/*	Put torch on wall support						 */
/*												     */
/*****************************************************/
void PutTorchOnSupport()
{
    SetNoTorchInHand();
    gTorchHand.burnout = TORCH_DURATION;
}

/*****************************************************/
/*													 */
/*	Light torch Room								 */
/*												     */
/*****************************************************/
void LightRoomTorch(u8 dir)
{
    u8 torchesRoom = gCurrentRoom->type_light_torch;

    if (dir & (LEFT|DOWN))
    {
        if (HAS_TORCH_LEFT(torchesRoom))
        {
            BOOL hasLeftBurnish = HAS_TORCH_LEFT_BURNISH(torchesRoom);

            if (gTorchHand.lighted ^ hasLeftBurnish)
            {
                if (hasLeftBurnish)
                    gTorchHand.lighted = TRUE;                    
                else
                    torchesRoom |= TORCH_LEFT_BURNISH;

                PlayFlame();
            }
        }
        else if (HAS_TORCH_SUPPORT_LEFT(torchesRoom))
        {
            torchesRoom |= gTorchHand.lighted ? TORCH_LEFT_BURNISH : TORCH_LEFT_EXTING;
            PutTorchOnSupport();
        }
    }
    else
    {
        if (HAS_TORCH_RIGHT(torchesRoom))
        {
            BOOL hasRightBurnish = HAS_TORCH_RIGHT_BURNISH(torchesRoom);

            if (gTorchHand.lighted ^ hasRightBurnish)
            {
                if (hasRightBurnish)
                    gTorchHand.lighted = TRUE;                    
                else
                    torchesRoom |= TORCH_RIGHT_BURNISH;

                PlayFlame();
            }            
        }
        else if (HAS_TORCH_SUPPORT_RIGHT(torchesRoom))
        {
            torchesRoom |= gTorchHand.lighted ? TORCH_RIGHT_BURNISH : TORCH_RIGHT_EXTING;
            PutTorchOnSupport();
        }
    }

    gCurrentRoom->type_light_torch = torchesRoom;

    if (IsRoomLit())
        DrawTorch();

    LightTorchInv();
}

/*****************************************************/
/*													 */
/*	Get limit horizontal							 */
/*												     */
/*****************************************************/
u8 GetLimitHoriz(u8 roomY, u8 pos)
{
    return _limitHoriz[roomY][pos];
}

/*****************************************************/
/*												     */
/*	Clean room										 */
/*												     */
/*****************************************************/
void CleanRoom(u8 cy)
{
    u8 i;
    for (i = 0; i < cy; i++)
    {
        u8* vmem = GetScreenPtr(0, i);
        cpct_memset(vmem, BACKGROUND_BYTE, ROOM_CX);
    }
}