//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** CPCTelera includes */
#include <cpctelera.h>
#include "../wincpctelera/wincpctelera_helper.h"

/** Structures includes */
#include "../structures/structures.h"

/** Game includes */
#include "../player/player.h"
#include "../room/room.h"
#include "../room/roomDrawItems.h"
#include "../room/roomDrawForest.h"
#include "../room/roomDrawCastle.h"
#include "../world/world.h"
#include "../video/video.h"
#include "../tools/tools.h"

/** Asm function includes */
#include "../asm/drawSpriteClipped.h"

/** Decors Sprites */
#include "../sprites/functions/packedSprites.h"
#include "../sprites/allSprites.h"

/** Local variables */
u8 _animationTimer;

/** external variables */
extern u8 gTimer;

/*****************************************************/
/*												     */
/*	Draw forest Castle								 */
/*												     */
/*****************************************************/
void DrawForestCastle()
{
    #define POS_CASTLE_X	(SCREEN_CX - CASTLE_W) / 2
    #define POS_CASTLE_Y	12

    u8* spriteEntrance;

    u8* vmem = cpctm_screenPtr(CPCT_VMEM_START, POS_CASTLE_X, POS_CASTLE_Y);
    DrawPackedSpriteFlip(G_CASTLE_INDEX, vmem, CASTLE_W, CASTLE_H, FALSE);

    vmem = cpctm_screenPtr(CPCT_VMEM_START, POS_CASTLE_X + 5, POS_CASTLE_Y + 1);
    DrawPackedSpriteMaskedFlip(G_TOWER_INDEX, vmem, TOWER_W, TOWER_H, FALSE);

    vmem = cpctm_screenPtr(CPCT_VMEM_START, POS_CASTLE_X + 10, POS_CASTLE_Y + 7);
    DrawPackedSpriteMaskedFlip(G_TOWER_INDEX, vmem, TOWER_W, TOWER_H - 6, FALSE);

    vmem = cpctm_screenPtr(CPCT_VMEM_START, POS_CASTLE_X + 14, POS_CASTLE_Y + 1);
    DrawPackedSpriteMaskedFlip(G_TOWER_INDEX, vmem, TOWER_W, TOWER_H, FALSE); 

    vmem = cpctm_screenPtr(CPCT_VMEM_START, POS_CASTLE_X + 8, POS_CASTLE_Y + 39);
    spriteEntrance = GetDecompressPackedSprite(G_ENTRANCE_INDEX);
    DrawSpriteMasked(spriteEntrance, vmem, ENTRANCE_W, ENTRANCE_H);
    FlipAndShiftSprite(spriteEntrance, ENTRANCE_W, ENTRANCE_H, LEFT);
    DrawSpriteMasked(spriteEntrance, vmem + ENTRANCE_W - 1, ENTRANCE_W, ENTRANCE_H);
}

/*****************************************************/
/*												     */
/*	Draw forest ground								 */
/*												     */
/*****************************************************/
void DrawForestGround()
{
    #define NB_FOREST_HERB	20

    u8* vmem = cpctm_screenPtr(CPCT_VMEM_START, 0, BUSH_H - 1);

    u8 i;
    for (i = 0; i < GROUND_FOREST_TILES_NB * 2; i++)
        DrawPackedSpriteFlip(G_ground_forest_tileset[ModuloPowerTwo(i, 2)], vmem + i * GROUND_FOREST_W, GROUND_FOREST_W, GROUND_FOREST_H, FALSE);

    for (i = 0; i < NB_FOREST_HERB; i++)
    {
        u8 x = GetRandMax(SCREEN_CX - HERB_W);
        u8 y = GetRandMax(ROOM_CY - BUSH_H - HERB_H) + BUSH_H;
        u8 herbIndex = GetRandMax(HERB_TILES_NB - 1);

        DrawPackedSpriteFlip(G_herb_tileset[herbIndex], GetScreenPtr(x, y), HERB_W, HERB_H, GetRandBool());
    }
}

/*****************************************************/
/*												     */
/*	Draw Background Forest							 */
/*												     */
/*****************************************************/
void DrawForestBackground()
{
    u8* vMem = cpctm_screenPtr(CPCT_VMEM_START, 0, 0);
    u8* bush = GetDecompressPackedSprite(G_BUSH_0_INDEX);
    
    BOOL roomToCastle = (gRoomId == TO_CASTLE_ROOM);
    if (roomToCastle)
    {
        cpct_drawSprite(bush, vMem, BUSH_W, BUSH_H);
        asm_drawSpriteClipped(SCREEN_CX - BUSH_W * BUSH_TILES_NB, bush, BUSH_W, BUSH_H, vMem + BUSH_W * BUSH_TILES_NB);

        bush = GetDecompressPackedSprite(G_BUSH_1_INDEX);
        cpct_drawSprite(bush, vMem + BUSH_W, BUSH_W, BUSH_H);
    }
    else
    {
        u16 offset = GetRandMax(5) + 1;        
        asm_drawSpriteClipped(offset, bush + BUSH_W - offset, BUSH_W, BUSH_H, vMem);
        cpct_drawSprite(bush, vMem + offset + BUSH_W, BUSH_W, BUSH_H);

        bush = GetDecompressPackedSprite(G_BUSH_1_INDEX);
        cpct_drawSprite(bush, vMem + offset, BUSH_W, BUSH_H);
        asm_drawSpriteClipped(SCREEN_CX - BUSH_W * BUSH_TILES_NB - offset, bush, BUSH_W, BUSH_H, vMem + BUSH_W * BUSH_TILES_NB + offset);
    }

    if (roomToCastle)
        DrawForestCastle();
}

/*****************************************************/
/*												     */
/*	Draw Lac										 */
/*												     */
/*****************************************************/
void DrawLac()
{
    u8* vmem = cpctm_screenPtr(CPCT_VMEM_START, 0, BUSH_H - 3);
    DrawPackedSpriteFlip(G_LAC_FOREST_INDEX, vmem, LAC_FOREST_W, LAC_FOREST_H, FALSE);
}

/*****************************************************/
/*												     */
/*	Draw Water Reflection							 */
/*												     */
/*****************************************************/
void DrawWaterReflection()
{   
   static const u8 sReflectColor[] =	{	
                                            cpctm_px2byteM0(COLOR_BRIGHT_WHITE, COLOR_BRIGHT_CYAN),
                                            cpctm_px2byteM0(COLOR_BRIGHT_CYAN, COLOR_BRIGHT_WHITE),
                                            cpctm_px2byteM0(COLOR_BRIGHT_CYAN, COLOR_BRIGHT_CYAN),
                                            cpctm_px2byteM0(COLOR_BRIGHT_WHITE, COLOR_SKY_BLUE),
                                            cpctm_px2byteM0(COLOR_SKY_BLUE, COLOR_BRIGHT_WHITE),
                                        };

   u8 animationTimer = _animationTimer;
   u8 modulo = ModuloPowerTwo(animationTimer, 2);
            
    u8 i;
    SetRandSeed(gRoomId + modulo);

    for (i = 0; i < (LAC_FOREST_H/2 - 2) / 2; i++)
    {
        u8 x = (GetRandMax(LAC_FOREST_W - 4) + i) % (LAC_FOREST_W - 4);
        u8 y = BUSH_H + 4*(i + modulo);

        u8* vmem = GetScreenPtr(x, y);
        *vmem = sReflectColor[(animationTimer + i) % sizeof(sReflectColor)];
    }

    _animationTimer++;
}
/*****************************************************/
/*												     */
/*	Draw Forest										 */
/*												     */
/*****************************************************/
void DrawRoomForest()
{
    if (!IsGameWin())
    {
        DrawForestBackground();
        DrawForestGround();
        DrawWallItems();

        if (gRoomId == LAC_FOREST_ROOM)
            DrawLac();
    }
    else
        CleanRoom(ROOM_CY);
}