//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** CPCTelera includes */
#include <cpctelera.h>
#include <stdio.h>

/** Structures includes */
#include "../structures/structures.h"

/** Game includes */
#include "../sound/sound.h"
#include "../video/video.h"
#include "../sprites/functions/moveSprite.h"
#include "../player/playerDraw.h"
#include "../player/playerMove.h"
#include "../player/player.h"
#include "../tools/tools.h"

/** Asm functions */
#include "../asm/spriteColorize.h"

/** Decors Sprites */
#include "../sprites/functions/packedSprites.h"
#include "../sprites/allSprites.h"

/** Room include */
#include "room.h"
#include "../world/world.h"

/** Extern variables */
extern SCoord gPlayerPos;
extern u8 gColorSky;

/** Local definitions */
#define NB_ROOM_INTERACT	2
#define BACKGROUND_SZ		(NB_ROOM_ITEMS * OBJECTS_SZ)
#define DEPTH_ROOM			29
#define SKY_CX				8
#define SKY_CY				38
#define SKY_OFFSET          15
#define WIN_OFF_Y	        15
#define DECO_Y              (MIDDLE_ROOM_Y - 30)
#define DECO_L_L_X	        8
#define DECO_L_R_X	        (ROOM_CX - DECO_L_L_X)

/** Local structures */
typedef struct
{
    u8 x, y;
    u8* background;
    u8* sprite;
    u8* item;
} SItemDraw;

typedef struct
{
    u8 x, xCx;
    u8 y, yCy;
    u8* item;
} SItemArea;

/** Local variables */
SItemArea _itemsCoord[NB_ROOM_ITEMS * NB_ROOM_ITEMS];

u8 _nbItemsDraw;
SItemDraw _itemsDraw[NB_ROOM_INTERACT];
u8* _itemBackground;
u8 _background[BACKGROUND_SZ];

u8  _chestx, _chesty;
SWidth _decoCenterCoord[NB_ROOM_INTERACT];

/** Local function */
void AddInteractiveItem(u8 x, u8 y, u8* item, u8* sprite);
void DrawInteractiveItems(BOOL init);

/*****************************************************/
/*												     */
/*	Initialize rooms items  						 */
/*												     */
/*****************************************************/
void InitDrawRoom()
{
    _nbItemsDraw = 0;
    _itemBackground = _background;

     cpct_memset(&_itemsCoord, NOTHING, sizeof(_itemsCoord));
}

/*****************************************************/
/*												     */
/*	Get items width									 */
/*												     */
/*****************************************************/
u8* GetCollideGroundItem()
{
    #define COLLIDE_OFFSET_Y	10

    u8 i;
    u8 dir = GetPlayerDir();

    u8 playerXCx;
    u8 playerYCy;

    u8 playerX = gPlayerPos.x + PLAYER_WIDTH / 2;
    u8 playerY = gPlayerPos.y + PLAYER_SPRITE_CY - COLLIDE_OFFSET_Y;
   
    SItemArea* itemCoord = _itemsCoord;
    
    if (HAS_MOVE_Y_AXIS(dir))
        playerY += (dir == TOP) ? -1 : 1;
    else
        playerX += (dir == LEFT) ? -1 : 1;

    playerXCx = playerX + PLAYER_WIDTH;
    playerYCy = playerY + COLLIDE_OFFSET_Y;

    for (i = 0; i < NB_ROOM_ITEMS * NB_ROOM_ITEMS; i++)
    {
        u8* item = itemCoord->item;
        if (item != NULL)
        {
            u8 itemX = itemCoord->x;
            u8 itemY = itemCoord->y;

            if (((itemX <= playerX) ? (itemCoord->xCx >= playerX) : (playerXCx >= itemX)) && 
                ((itemY <= playerY) ? (itemCoord->yCy >= playerY) : (playerYCy >= itemY)))
            {
                return item;
            }
        }

        itemCoord++;
    }
    return NULL;
}

/*****************************************************/
/*												     */
/*	Get number of Decors							 */
/*												     */
/*****************************************************/
u8 GetNbDecors(u8* wallFront)
{
    u8 nbDecors = 0;
    for (u8 i = DECO_CENTER_1; i <= DECO_CENTER_3; i++)
    {
        if (wallFront[i] != NO_DECO)
            nbDecors++;
    }

    return nbDecors;
}

/*****************************************************/
/*												     */
/*	Get Decors Center coordinates					 */
/*												     */
/*****************************************************/
SWidth* GetDecoCenterCoord()
{
    return _decoCenterCoord;
}

/*****************************************************/
/*												     */
/*	Colorize tapestry with color table				 */
/*												     */
/*****************************************************/
void ColorizeTapestry(u8* sprite, u8* sColor)
{
    u8 i;
    for (i = 0; i < 3; i++)
        asm_spriteColorize(i + 2, sColor[i], TAPRESTRY_F_W, TAPRESTRY_F_H, sprite);
}

/*****************************************************/
/*												     */
/*	Colorize tapestry								 */
/*												     */
/*****************************************************/
void ColorTapestryType(u8 decoId, u8* sprite)
{
    static const u8 sRedColor[] = { 5, 6, 9 };
    static const u8 sOrangeColor[] = { 6, 8, 12 };

    if (decoId != WALL_TAPESTRY_BLUE)
        ColorizeTapestry(sprite, decoId == WALL_TAPESTRY_RED ? sRedColor : sOrangeColor);
}

/*****************************************************/
/*												     */
/*	Draw Decors	Center								 */
/*												     */
/*****************************************************/
void DrawDecorsCenter(u8* wallItems, u8 nbDecorsCenter)
{
    u8 offset;
    u8 decoIndex = 0;

    u8 cx = ROOM_CX;
    u8 x = 0;
    u8 y = (gRoomSize == ROOM_LARGE) ? _posFrontWall : _posFrontWall + 18;

    if (_leftWall)
    {
        x = _posLeftAngle;
        cx -= _posLeftAngle;
    }

    if (_rightWall)
    {
        cx -= _posLeftAngle;
    }

    offset = cx / (nbDecorsCenter + 1);
    x += offset;

    while (nbDecorsCenter != 0)
    {
        u8* decoPtr = &wallItems[DECO_CENTER_1 + decoIndex];
        u8 deco = *decoPtr;
        if (deco != NO_DECO)
        {
            u8* vmem;
            u8 centerX = 0;

            if (deco >= WALL_TAPESTRY_BLUE && deco <= WALL_TAPESTRY_ORANGE)
            {
                u8* sprite = GetDecompressPackedSprite(G_TAPRESTRY_F_INDEX);

                centerX = x - TAPRESTRY_F_W/2;
                vmem = GetScreenPtr(centerX, y + 2);
                cx = TAPRESTRY_F_W;		

                ColorTapestryType(deco, sprite);
                DrawSpriteMasked(sprite, vmem, TAPRESTRY_F_W, TAPRESTRY_F_H);
            }
            else if (deco == WALL_FOREST_SIGN)
            {
                BOOL leftSign = (decoIndex == 0);
                vmem = GetScreenPtr(leftSign ? x - 10 : x + 4 , y + 72);
                DrawPackedSpriteMaskedFlip(G_SIGN_INDEX, vmem, SIGN_W, SIGN_H, leftSign);
            }
            else if (deco == WALL_WINDOW)
            {
                centerX = x - WINDOW_F_W;

                vmem = GetScreenPtr(centerX + 2, y + 8);
                cpct_drawSolidBox(vmem, gColorSky, SKY_CX, SKY_CY);

                vmem = GetScreenPtr(centerX, y + 2);
                DrawMirrorPackedSpriteMasked(G_WINDOW_F_INDEX, vmem, WINDOW_F_W, WINDOW_F_H);

                cx = WINDOW_F_W * 2;
            }
            else if (deco >= WALL_SHIELD_0 && deco <= WALL_SHIELD_2)
            {
                centerX = x - SHIELDS_W;
                vmem = GetScreenPtr(centerX, y + 14);
                DrawMirrorPackedSpriteMasked(G_shields_tileset[deco - WALL_SHIELD_0], vmem, SHIELDS_W, SHIELDS_H);

                cx = SHIELDS_W * 2;
            }
            else if (deco == WALL_FIREPLACE || deco == WALL_FIREPLACE_KEY)
            {
                centerX = x - FIREPLACE_W;
                vmem = GetScreenPtr(centerX, y + 34);

                cx = FIREPLACE_W * 2;
                DrawMirrorPackedSpriteMasked(G_FIREPLACE_INDEX, vmem, FIREPLACE_W, FIREPLACE_H);

                if (deco == WALL_FIREPLACE_KEY)
                {
                    u8* sprite = GetPackedSprite(G_objects_tileset[METAL_KEY]);
                    centerX = 42;
                    cx = 2;
                    AddInteractiveItem(centerX, y + 29, decoPtr, sprite);      
                }
            }
            else if (deco >= WALL_ALTAR && deco <= WALL_ALTAR_SCEPTER)
            {				
                #define ALTAR_OBJECT_OFFSET     71

                centerX = x - ALTAR_W;
                vmem = GetScreenPtr(centerX, y + 52);

                cx = ALTAR_W * 2;
                DrawMirrorPackedSpriteMasked(G_ALTAR_INDEX, vmem, ALTAR_W, ALTAR_H);

                if (deco != WALL_ALTAR)
                {
                    u8* sprite = GetPackedSprite(G_objects_tileset[deco == WALL_ALTAR_GOLD_CUP ? GOLDEN_CUP : deco - ALTAR_OBJECT_OFFSET]);                
                    cx = 4;
                    AddInteractiveItem(centerX + 1, y + 45, decoPtr, sprite);             
                }
            }
            else if (deco == WALL_CHAINS)
            {
                u8 i;
                centerX = x - CHAINS_W / 2;

                for (i = 0; i < GetRandMaxTwo() + 2; i++)
                {
                    vmem = GetScreenPtr(centerX, y + i * CHAINS_H);
                    DrawPackedSpriteMaskedFlip(G_CHAINS_INDEX, vmem, CHAINS_W, CHAINS_H, FALSE);
                }
            }
            else if (deco >= WALL_PAINT_0 && deco <= WALL_PAINT_3)
            {
                centerX = x - PAINT_FRAME_W / 2;
                vmem = GetScreenPtr(centerX, y);
                DrawPackedSpriteFlip(G_PAINT_FRAME_INDEX, vmem, PAINT_FRAME_W, PAINT_FRAME_H, FALSE);

                cx = PAINTS_H;
                vmem = GetScreenPtr(centerX, y + 3);
                DrawPackedSpriteFlip(G_paints_tileset[deco - WALL_PAINT_0], vmem + 1, PAINTS_W, PAINTS_H, FALSE);
            }

            x += offset;
            _decoCenterCoord[decoIndex].x = centerX;
            _decoCenterCoord[decoIndex].cx = centerX + cx;
            nbDecorsCenter--;
        }
        else
        {
            _decoCenterCoord[decoIndex].cx = 0;
        }

        decoIndex++;		
    }
}

/*****************************************************/
/*												     */
/*	Draw Decors	Left								 */
/*												     */
/*****************************************************/
void DrawDecorsLeft()
{
    u8 leftDeco = gWallLatt[DECO_LAT_LEFT];
    if (leftDeco != NO_DECO)
    {
        u8 x = DECO_L_L_X;
        u8 y = DECO_Y;

        u8 door = gCurrentRoom->doorDef[DOOR_LEFT];
        u8 doorType = GET_DOOR_TYPE(door);

        if (doorType != DOOR_TYPE_WALL)
        {
            x += 3;
            y -= 8;
        }

        if (leftDeco == WALL_SECRET_KEY_HOLE)
        {
            #define SECRET_HOLE_X (DECO_L_L_X + 4)
            #define SECRET_HOLE_Y (DECO_Y + 63)

            u8* vmem = cpctm_screenPtr(CPCT_VMEM_START, SECRET_HOLE_X, SECRET_HOLE_Y);
            cpct_drawSprite(secret_key_hole, vmem, SECRET_KEY_HOLE_W, SECRET_KEY_HOLE_H);
        }
        else if (leftDeco == WALL_WINDOW)
        {
            #define POS_X_L_WINDOW   (x)
            #define POS_Y_L_WINDOW   (y + WIN_OFF_Y)

            cpct_drawSolidBox(GetScreenPtr(POS_X_L_WINDOW + 1, POS_Y_L_WINDOW + SKY_OFFSET), gColorSky, SKY_CX, SKY_CY);
            DrawPackedSpriteMaskedFlip(G_WINDOW_L_INDEX, GetScreenPtr(POS_X_L_WINDOW, POS_Y_L_WINDOW), WINDOW_L_W, WINDOW_L_H, FALSE);
        }
        else
        {
            u8* sprite = GetDecompressPackedSprite(G_TAPRESTRY_L_INDEX);
            ColorTapestryType(leftDeco, sprite);

            DrawSpriteMasked(sprite, GetScreenPtr(x, y), TAPRESTRY_L_W, TAPRESTRY_L_H);
        }
    }
}

/*****************************************************/
/*												     */
/*	Draw Decors Right								 */
/*												     */
/*****************************************************/
void DrawDecorsRight()
{
    u8 rightDeco = gWallLatt[DECO_LAT_RIGHT];
    if (rightDeco != NO_DECO)
    {
        u8 x = DECO_L_R_X;
        u8 y = DECO_Y;

        u8 door = gCurrentRoom->doorDef[DOOR_RIGHT];
        u8 doorType = GET_DOOR_TYPE(door);
        if (doorType != DOOR_TYPE_WALL)
        {
            x -= 3;
            y -= 8;
        }

        if (rightDeco == WALL_WINDOW)
        {
            #define POS_X_R_WINDOW   (x - WINDOW_L_W)
            #define POS_Y_R_WINDOW   (y + WIN_OFF_Y)

            cpct_drawSolidBox(GetScreenPtr(POS_X_R_WINDOW + 1, POS_Y_R_WINDOW + SKY_OFFSET), gColorSky, SKY_CX, SKY_CY);
            DrawPackedSpriteMaskedFlip(G_WINDOW_L_INDEX, GetScreenPtr(POS_X_R_WINDOW, POS_Y_R_WINDOW), WINDOW_L_W, WINDOW_L_H, TRUE);
        }
        else
        {
            u8* sprite = GetDecompressSpriteFlip(GetPackedSprite(G_TAPRESTRY_L_INDEX), TAPRESTRY_L_W, TAPRESTRY_L_H, TRUE);
            ColorTapestryType(rightDeco, sprite);

            DrawSpriteMasked(sprite, GetScreenPtr(x - TAPRESTRY_L_W, y), TAPRESTRY_L_W, TAPRESTRY_L_H);
        }
    }
}

/*****************************************************/
/*												     */
/*	Draw Decors	on Wall								 */
/*												     */
/*****************************************************/
void DrawWallItems()
{
    u8* wallFront = gCurrentRoom->wallFront;
    u8 nbDecorsCenter = GetNbDecors(wallFront);

    if (nbDecorsCenter != 0)
        DrawDecorsCenter(wallFront, nbDecorsCenter);

    DrawDecorsLeft();
    DrawDecorsRight();
}

/*****************************************************/
/*													 */
/*	Draw Item in background player buffer			 */
/*												     */
/*****************************************************/
BOOL DrawInPlayerBuffer(u8 itemX, u8 itemY, u8 spriteCx, u8 spriteCy, u8* sprite)
{
    u8 posx = 0;
    u8 cx = 0;
    u8 spritexCx;

    i8 x = itemX - gPlayerPos.x;
    spritexCx = spriteCx + x;

    /** Left redraw */
    if (x < 0)
    {
        cx = spritexCx;
    }
    else
    {
        posx = x;

        /** Full redraw */
        if (posx <= PLAYER_SPRITE_CX - spriteCx)
            cx = spriteCx;
        else 
        /** Right redraw */
        if (spritexCx > PLAYER_SPRITE_CX)
            cx = spriteCx - (spritexCx -  PLAYER_SPRITE_CX);
    }

    if (cx != 0)
    {
        u8 i;
        u8* vmem = GetScreenPtr(itemX, itemY);
        u8* backgroundIn = cpctm_spriteBufferPtr(GetPlayerBackgroundBuffer(), PLAYER_SPRITE_CX, posx, (u8)(itemY - gPlayerPos.y));
        u8* spriteDraw = (x < 0) ? sprite + (spriteCx - cx) : sprite;

        if (itemY < gPlayerPos.y)
        {
            cpct_drawSprite(sprite, vmem, spriteCx, spriteCy);
            return TRUE;
        }
        else if (cx != spriteCx)
        {
            DrawSpriteMasked(sprite, vmem, spriteCx, spriteCy);
        }

        /**	Draw partial into sprite buffer of player */
        for (i = 0; i < spriteCy; i++)
            cpct_drawToSpriteBufferMaskedAlignedTable(PLAYER_SPRITE_CX, backgroundIn + i * PLAYER_SPRITE_CX, cx, 1, spriteDraw + i * spriteCx, gMaskTable);

        return TRUE;
    }
    return FALSE;
}

/*****************************************************/
/*												     */
/*	Add Item coordinates							 */
/*												     */
/*****************************************************/
u8* AddItemCoord(u8 index, u8* item, u8 x, u8 y, u8 cx, u8 cy)
{
    SItemArea* itemArea = &_itemsCoord[index];

    itemArea->x = x;
    itemArea->xCx = x + cx;
    itemArea->y = y;
    itemArea->yCy = y + cy;
    itemArea->item = item;

    return GetScreenPtr(x, y);
}

/*****************************************************/
/*												     */
/*	Draw chest opened								 */
/*												     */
/*****************************************************/
void DrawChestOpened()
{
    if (HasLightInRoom())
    {
        u8* spriteChestOpen = GetDecompressPackedSprite(G_CHEST_OPEN_INDEX);
        DrawInPlayerBuffer(_chestx, _chesty, CHEST_OPEN_W, CHEST_OPEN_H, spriteChestOpen);
    }

    PlayDoor();
}

/*****************************************************/
/*												     */
/*	Draw Item										 */
/*												     */
/*****************************************************/
void DrawRoomItem(u8 itemPos, u8 x, u8 y, u8* item)
{
    #define POS_CHEST_Y		(9 - CHEST_H - 2)
    #define OFFSET_TRAP		5
    #define	ROOM_EVENT_CX	12
    #define	ROOM_EVENT_CY	20

    if (item != NULL)
    {
        u8 itemId = *item;
        if (itemId == ROOM_TABLE)
        {
            u8* vmem = AddItemCoord(itemPos, item, x, y - 15, TABLE_W * 2, TABLE_H);
            DrawMirrorPackedSpriteMasked(G_TABLE_INDEX, vmem, TABLE_W, TABLE_H);
        }
        else if (itemId == ROOM_ITEM_TRAP || itemId == ROOM_ITEM_HIDE_TRAP)
        {
            u8* vmem = AddItemCoord(itemPos, item, x, y + OFFSET_TRAP, TRAP_W * 2, TRAP_H);
            if (itemId == ROOM_ITEM_TRAP)
                DrawMirrorPackedSpriteMasked(G_TRAP_INDEX, vmem, TRAP_W, TRAP_H);
            else
                DrawPackedSpriteMaskedFlip(G_TRAP_HIDDEN_INDEX, vmem, TRAP_HIDDEN_W, TRAP_HIDDEN_H, FALSE);
        }
        else if (itemId == ROOM_CHEST_OPEN || itemId == ROOM_CHEST_CLOSE)
        {      
            u8* vmem = AddItemCoord(itemPos, item, x, y + 9, CHEST_W, CHEST_H);

            u16 chestSpriteIndex = G_CHEST_OPEN_INDEX;
            DrawPackedSpriteMaskedFlip(G_CHEST_INDEX, vmem , CHEST_W, CHEST_H, FALSE);
            
            vmem = GetScreenPtr(x, y + POS_CHEST_Y);
            if (itemId == ROOM_CHEST_CLOSE)
            {
                _chestx = x;
                _chesty = y + POS_CHEST_Y;
                chestSpriteIndex = G_CHEST_CLOSE_INDEX;
            }
   
            DrawPackedSpriteMaskedFlip(chestSpriteIndex, vmem, CHEST_OPEN_W, CHEST_OPEN_H, FALSE);
        }
        else if (itemId == ROOM_SKELETON)
        {
            u8* vmem = AddItemCoord(itemPos, item, x, y + 4, SKELETON_W, SKELETON_H);
            DrawPackedSpriteMaskedFlip(G_SKELETON_INDEX, vmem, SKELETON_W, SKELETON_H, GetRandBool());
        }
        else if (itemId == ROOM_DEMON_CIRCLE)
        {
            u8* vmem = AddItemCoord(itemPos, item, x, y + 8, CIRCLE_W * 3, CIRCLE_H);
            DrawMirrorPackedSpriteMasked(G_CIRCLE_INDEX, vmem, CIRCLE_W, CIRCLE_H);
        }
        else if (itemId == ROOM_EVENT_LIGHT_OFF)
        {
            AddItemCoord(itemPos, item, x, y, ROOM_EVENT_CX, ROOM_EVENT_CY);			
        }
    }
}

/*****************************************************/
/*												     */
/*	Draw Items interactive							 */
/*												     */
/*****************************************************/
void DrawInteractiveItems(BOOL init)
{
    if (HasLightInRoom())
    {
        u8* temp = GET_TEMP_LOC_END - OBJECTS_SZ;

        SItemDraw* itemDraw = _itemsDraw;
        u8 i;

        for (i = 0; i < _nbItemsDraw; i++)
        {
            if (itemDraw->item != NULL)
            {
                u8 itemId = *itemDraw->item;
                u8 x = itemDraw->x;
                u8 y = itemDraw->y;

                cpct_memcpy(temp, itemDraw->background, OBJECTS_SZ);

                if (itemId != NOTHING && (itemId != WALL_FIREPLACE && itemId != WALL_ALTAR) && itemDraw->sprite != NULL)
                {
                    u8* sprite = Decompress(itemDraw->sprite);
                    cpct_drawToSpriteBufferMaskedAlignedTable(OBJECTS_W, temp, OBJECTS_W, OBJECTS_H, sprite, gMaskTable);
                }

                if (init)
                {
                    u8* vmem = GetScreenPtr(x, y);
                    cpct_drawSprite(temp, vmem, OBJECTS_W, OBJECTS_H);
                }
                else
                {
                    DrawInPlayerBuffer(x, y, OBJECTS_W, OBJECTS_H, temp);
                }
            }

            itemDraw++;
        }
    }
}

/*****************************************************/
/*												     */
/*	Draw Item										 */
/*												     */
/*****************************************************/
void AddInteractiveItem(u8 x, u8 y, u8* itemId, u8* sprite)
{
    u8* vmem = GetScreenPtr(x, y);
    SItemDraw* itemDraw = &_itemsDraw[_nbItemsDraw++];

    itemDraw->item = itemId;
    itemDraw->x = x;
    itemDraw->y = y;
    itemDraw->sprite = sprite;

    itemDraw->background = _itemBackground;
    _itemBackground += OBJECTS_SZ;
    cpct_getScreenToSprite(vmem, itemDraw->background, OBJECTS_W, OBJECTS_H);
}

/*****************************************************/
/*												     */
/*	Set all Items to be drawn						 */
/*												     */
/*****************************************************/
void InitInteractiveItems()
{
    SItemsRoom* roomItems = GetRoomItems(gRoomId);
    if (roomItems != NULL)
    {
        u8* items = roomItems->items;

        u8 x, y;
        for (y = 0; y < NB_ROOM_ITEMS; y++)
        {
            u8 width = (GetLimitHoriz(y, LIMIT_CX)) / NB_ROOM_ITEMS;
            u8* item = &items[y * NB_ROOM_ITEMS];

            for (x = 0; x < NB_ROOM_ITEMS; x++)
            {
                u8 itemId = *item;
                if (itemId != NOTHING && itemId < ROOM_STATIC)
                {
                    u8 startX = GetLimitHoriz(y, LIMIT_LEFT) + width / 2;
                    u8 xItem = startX + width * x;

                    u8 yItem = y * DEPTH_ROOM + OUT_ROOM_Y + 6;
                    u8* sprite = GetPackedSprite(GetObjectSpriteId(itemId));

                    AddItemCoord(x + y * NB_ROOM_ITEMS, item, xItem, yItem, OBJECTS_W, OBJECTS_H);
                    AddInteractiveItem(xItem, yItem, item, sprite);
                }

                item++;
            }
        }
    }
}

/*****************************************************/
/*												     */
/*	Draw decor Items								 */
/*												     */
/*****************************************************/
void DrawDecorItems()
{
    SItemsRoom* roomItems = GetRoomItems(gRoomId);
    if (roomItems != NULL)
    {
        u8* items = roomItems->items;

        u8 x, y;
        for (y = 0; y < NB_ROOM_ITEMS; y++)
        {
            u8 width = GetLimitHoriz(y, LIMIT_CX) / NB_ROOM_ITEMS;
            u8 startX = GetLimitHoriz(y, LIMIT_LEFT) + width / 2;

            u8* item = &items[y * NB_ROOM_ITEMS];
            u8 itemIndex = y * NB_ROOM_ITEMS;
            u8 nbItems = 0;

            for (x = 0; x < NB_ROOM_ITEMS; x++)
            {
                _itemsCoord[itemIndex + x].item = NULL;
                if (*item >= ROOM_STATIC)
                    DrawRoomItem(itemIndex + x, startX + width * nbItems, OUT_ROOM_Y + y * DEPTH_ROOM, item);

                nbItems++;
                item++;
            }
        }
    }

    if (_leftWall && (ModuloPowerTwo(gRoomId, 2) == 0))
    {
        u8* vmem = cpctm_screenPtr(CPCT_VMEM_START, 0, 0);
        DrawPackedSpriteMaskedFlip(G_WEB_INDEX, vmem, WEB_W, WEB_H, FALSE);
    }
    else
    if (_rightWall && (gRoomId % 5 == 0))
    {
        u8* vmem = cpctm_screenPtr(CPCT_VMEM_START, SCREEN_CX - WEB_W, 0);
        DrawPackedSpriteMaskedFlip(G_WEB_INDEX, vmem, WEB_W, WEB_H, TRUE);
    }
}

/*****************************************************/
/*												     */
/*	Draw room Items									 */
/*												     */
/*****************************************************/
void DrawRoomItems()
{
    DrawDecorItems();
    InitInteractiveItems();
    DrawInteractiveItems(TRUE);
}
