//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** CPCTelera includes */
#include <cpctelera.h>
#include "../../wincpctelera/wincpctelera_helper.h"

/** Asm includes */
#include "../../asm/copySpriteBufferPartial.h"
#include "../../asm/copySpriteBufferClipped.h"

/** Game includes */
#include "../../structures/structures.h"
#include "../../room/room.h"
#include "../../world/world.h"
#include "../../structures/structures.h"
#include "../../video/video.h"

/** Move Sprite include */
#include "./moveSprite.h"

extern void DrawPlayerToBuffer();
extern void RenderPlayer();

/** Local variables */
SCoord* _coord;
u8* _sprite;
u8* _backGround;
u8* _tempBackground;

/*****************************************************/
/*													 */
/*	Setup Sprite moving								 */
/*												     */
/*****************************************************/
void SetupMoveSprite(SMoveSprite* moveSprite, SCoord* coord)
{
    u8* vmem = GetScreenPtr(coord->x, coord->y);
    
    _coord = coord;
    _sprite = moveSprite->sprite;
    _backGround = moveSprite->backGround;

    _tempBackground = GET_TEMP_LOC_END - PLAYER_SPRITE_CY * 2;
    
    cpct_getScreenToSprite(vmem, _backGround, PLAYER_SPRITE_CX, PLAYER_SPRITE_CY);
}

/*****************************************************/
/*													 */
/*	Render Sprite									 */
/*												     */
/*****************************************************/
void RenderSprite()
{
    if (HasLightInRoom())
    {
        // Copy background
        cpct_memcpy(_backGround, _sprite, PLAYER_SPRITE_SZ);
        DrawPlayerToBuffer();
        RenderPlayer();
    }
}

/*****************************************************/
/*													 */
/*	Move Down Sprite								 */
/*												     */
/*****************************************************/
void MoveDownSprite(u8 offset)
{
    u8 x = _coord->x;
    u8 y = _coord->y;

    u8 cx = PLAYER_SPRITE_CX;
    u8 cy = PLAYER_SPRITE_CY;

    /** Restore screen background */
    cpct_memcpy(_tempBackground, _backGround, offset * cx);

    /** Offset Background in SpriteBuffer */
    cpct_drawToSpriteBuffer(cx, _sprite, cx, cy - offset, _backGround + offset * cx);

    /** Copy Screen next delta to Background position to SpriteBuffer */
    cpct_getScreenToSprite(GetScreenPtr(x, y + cy), _sprite + PLAYER_SPRITE_SZ - offset * cx, cx, offset);
    
    /** Update sprite position */
    _coord->y += offset;

    /** Render sprite */
    RenderSprite();

    /** Erase top trail */
    cpct_drawSprite(_tempBackground, GetScreenPtr(x, y), cx, offset);
}

/*****************************************************/
/*													 */
/*	Move Top Sprite	     							 */
/*												     */
/*****************************************************/
void MoveTopSprite(u8 offset)
{
    u8 x = _coord->x;
    u8 y = _coord->y;	
    
    u8 cx = PLAYER_SPRITE_CX;
    u8 cyMinusOffset = PLAYER_SPRITE_CY - offset;

    /** Restore screen background */
    cpct_memcpy(_tempBackground, _backGround + PLAYER_SPRITE_SZ - offset * cx, offset * cx);

    /** Offset Background in SpriteBuffer */
    cpct_drawToSpriteBuffer(cx, _sprite + offset * cx, cx, cyMinusOffset, _backGround);

    /** Copy Screen next delta to Background position to SpriteBuffer */
    cpct_getScreenToSprite(GetScreenPtr(x, y - offset), _sprite, cx, offset);

    /** Update sprite position */
    _coord->y -= offset;

    /** Render sprite */
    RenderSprite();

    /** Erase bottom trail */
    cpct_drawSprite(_tempBackground, GetScreenPtr(x, y + cyMinusOffset), cx, offset);
}
/*****************************************************/
/*													 */
/*	Move Right Sprite	     						 */
/*												     */
/*****************************************************/
void MoveRightSprite(u8 offset)
{
    u8 x = _coord->x;
    u8 y = _coord->y;
    
    u8 cx = PLAYER_SPRITE_CX;
    u8 cy = PLAYER_SPRITE_CY;
    u8 cxMinusOffset = cx - offset;

    u8* spriteDest = _sprite;
    u8* spriteSrc = _backGround;
    
    /** Restore screen background */
    asm_copySpriteBufferPartial(offset, cx, cy, _tempBackground, spriteSrc);

    /** Offset Background in SpriteBuffer */
    asm_copySpriteClipped(cxMinusOffset, cx, cy, spriteSrc + offset, spriteDest);

    /** Copy Screen next delta to Background position to SpriteBuffer */
    cpct_getScreenToSprite(GetScreenPtr(x + cx, y), spriteSrc, offset, cy);
    cpct_drawToSpriteBuffer(cx, spriteDest + cxMinusOffset, offset, cy, spriteSrc);

    /** Update sprite position */
    _coord->x += offset;

    /** Render sprite */
    RenderSprite();

    /** Erase right trail */
    cpct_drawSprite(_tempBackground, GetScreenPtr(x, y), offset, cy);
}

/*****************************************************/
/*													 */
/*	Move Left Sprite	     						 */
/*												     */
/*****************************************************/
void MoveLeftSprite(u8 offset)
{
    u8 x = _coord->x;
    u8 y = _coord->y;
    
    u8 cx = PLAYER_SPRITE_CX;
    u8 cy = PLAYER_SPRITE_CY;
    u8 cxMinusOffset = cx - offset;
    
    u8* spriteDest = _sprite;
    u8* spriteSrc = _backGround;

    /** Restore screen background */
    asm_copySpriteBufferPartial(offset, cx, cy, _tempBackground, spriteSrc + cxMinusOffset);

    /** Offset Background in SpriteBuffer */
    asm_copySpriteClipped(cxMinusOffset, cx, cy, spriteSrc, spriteDest + offset);

    /** Copy Screen next delta to Background position to SpriteBuffer */
    cpct_getScreenToSprite(GetScreenPtr(x - offset, y), spriteSrc, offset, cy);
    cpct_drawToSpriteBuffer(cx, spriteDest, offset, cy, spriteSrc);

    /** Update sprite position */
    _coord->x -= offset;

    /** Render sprite */
    RenderSprite();

    /** Erase left trail */
    cpct_drawSprite(_tempBackground, GetScreenPtr(x + cxMinusOffset, y), offset, cy);
}

/*****************************************************/
/*													 */
/*	Redraw Sprite into buffer  						 */
/*												     */
/*****************************************************/
void RedrawIntoBuffer()
{
    cpct_drawToSpriteBuffer(PLAYER_SPRITE_CX, _sprite, PLAYER_SPRITE_CX, PLAYER_SPRITE_CY, _backGround);

   //cpct_memset(_sprite, 0, PLAYER_SPRITE_SZ);
}

#ifdef WINCPCTELERA
/*****************************************************/
/*													 */
/*	Draw Sprite to screen for WinCPCTelera			 */
/*												     */
/*****************************************************/
void RenderSpriteWinCpctelera(u8* vmem, u8* sprite, u8 height)
{
    cpct_drawSprite(sprite, vmem, PLAYER_SPRITE_CX, height);
}
#endif