//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** CPCTelera includes */
#include <cpctelera.h>

/** Assembly function */
#include "../asm/spriteColorize.h"

/** Game includes */
#include "../structures/structures.h"
#include "../text/text.h"
#include "../video/video.h"
#include "../player/player.h"
#include "../player/playerDraw.h"
#include "../room/room.h"
#include "../world/world.h"
#include "../sound/sound.h"
#include "../input/input.h"
#include "../tools/tools.h"

/** Sprites includes */
#include "../sprites/functions/packedSprites.h"
#include "../sprites/allSprites.h"

/** UI include */
#include "uiboard.h"

/** Text includes */
#include "../text/fontsHelper.h"

/** Local defines */
#define ORG_INV_X		90/2
#define ORG_INV_Y		(BOARD_Y + 2)

#define INFO_LIFE_X		6	
#define INFO_LIFE_Y		SCREEN_CY - 35
#define INFO_GOLD_X		8
#define INFO_GOLD_Y		SCREEN_CY - 19

#define MAX_INV_X		4
#define MAX_INV_Y		1

#define HAND_Y			(BOARD_Y + 12)
#define LEFT_HAND_X		50/2
#define RIGHT_HAND_X	68/2

#define UNSELECTED		0
#define SELECTED		1

/** Local variables */
u8 _invSelection;
u8 _invX, _invY;
u8 _handX;
i8 _index;
static BOOL _selHand;

/** Local function definition */
void RedrawInventory();
void DrawHandSelection(u8 selection);

/*****************************************************/
/*													 */
/*	Draw selection					  				 */
/*												     */
/*****************************************************/
void DrawSelection(u8 selection)
{
    u8* vmem = GetScreenPtr(ORG_INV_X + _invX * (UI_SELECT_W + 1), ORG_INV_Y + _invY * (UI_SELECT_H + 1));
    u8* spriteSelect = GetDecompressPackedSprite(G_UI_SELECT_INDEX);

    if (selection != 0)
        asm_spriteColorize(1, 6, UI_SELECT_W, UI_SELECT_H, spriteSelect);

    DrawSpriteMasked(spriteSelect, vmem, UI_SELECT_W, UI_SELECT_H);
}

/*****************************************************/
/*													 */
/*	Redraw board					  				 */
/*												     */
/*****************************************************/
void RedrawBoard()
{
    RedrawInventory();

    DrawGold();
    DrawLife();
    DrawHandSelection(UNSELECTED);

    _handX = HAND_RIGHT;

    DrawHand(HAND_LEFT);
    DrawHand(HAND_RIGHT);
}

/*****************************************************/
/*													 */
/*	Close board						  				 */
/*												     */
/*****************************************************/
void CloseBoard()
{
    DrawSelection(UNSELECTED);

    _invX = 0;
    _invY = 0;
    _selHand = TRUE;
    _index = -1;
}

/*****************************************************/
/*													 */
/*	Draw selection					  				 */
/*												     */
/*****************************************************/
void DrawHandSelection(u8 selection)
{
    u8* vmem = GetScreenPtr((_handX == HAND_LEFT) ? LEFT_HAND_X - 1: RIGHT_HAND_X - 1, HAND_Y - 3);
    u8* spriteSelect = GetDecompressPackedSprite(G_UI_SELECT_LARGE_INDEX);

    if (selection != 0)
        asm_spriteColorize(1, 6, UI_SELECT_LARGE_W, UI_SELECT_LARGE_H, spriteSelect);

    DrawSpriteMasked(spriteSelect, vmem, UI_SELECT_LARGE_W, UI_SELECT_LARGE_H);
}

/*****************************************************/
/*													 */
/*	Get inventory object screen position			 */
/*												     */
/*****************************************************/
u8* GetScreenPosInv(u8 invPos)
{
    return GetScreenPtr(ORG_INV_X + invPos % (MAX_INV_X + 1) * (UI_SELECT_W + 1) + 1, ORG_INV_Y + invPos / (MAX_INV_X + 1) * (UI_SELECT_H + 1) + 2);
}

/*****************************************************/
/*													 */
/*	Draw inventory object				  			 */
/*												     */
/*****************************************************/
void DrawInventoryObject(u8* vmem, u8 objInv)
{
    DrawPackedSpriteFlip(G_OBJECTS_00_INDEX, vmem, OBJECTS_W, OBJECTS_H, FALSE);
    
    if (objInv != NOTHING)
    {
        DrawPackedSpriteMaskedFlip(GetObjectSpriteId(objInv), vmem, OBJECTS_W, OBJECTS_H, FALSE);

        if (objInv == GOLDEN_CUP && (gAdventure & ADV_CUP_FILLED))
            DrawSpriteMasked(calice_filled, vmem, CALICE_FILLED_W, CALICE_FILLED_H);
    }
}

/*****************************************************/
/*													 */
/*	Draw inventory slot					  			 */
/*												     */
/*****************************************************/
void DrawInventorySlot(u8 index)
{
    u8 objInv = gPlayer.inventory[index];
    u8* vmem = GetScreenPosInv(index);

    DrawInventoryObject(vmem, objInv);
}

/*****************************************************/
/*													 */
/*	Draw hand content				  				 */
/*												     */
/*****************************************************/
void DrawHand(u8 hand)
{
    u8* vmem = (hand == HAND_RIGHT) ? cpctm_screenPtr(CPCT_VMEM_START, RIGHT_HAND_X , HAND_Y) : cpctm_screenPtr(CPCT_VMEM_START, LEFT_HAND_X, HAND_Y);
    DrawInventoryObject(vmem, gPlayer.inHand[hand]);
}

/*****************************************************/
/*													 */
/*	Flip inventory with hand object					 */
/*												     */
/*****************************************************/
void FlipInventoryHand(u8 hand, u8 itemId)
{
    u8 inHandItemId = gPlayer.inHand[hand];
    if (inHandItemId == TORCH)
        ExtinguishTorch();

    gPlayer.inventory[_index] = inHandItemId;
    gPlayer.inHand[hand] = itemId;

    DrawInventorySlot(_index);
    DrawHand(hand);
    
    PlaySelect();
}

/*****************************************************/
/*													 */
/* Clean inventory slot								 */
/*												     */
/*****************************************************/
void CleanInventorySlot(u8 index)
{
    gPlayer.inventory[index] = NOTHING;
    DrawInventorySlot(index);
}

/*****************************************************/
/*													 */
/*	Get hand for object								 */
/*												     */
/*****************************************************/
u8 GetHandForObject(u8 objectId)
{
    return (objectId >= TORCH) ? HAND_RIGHT : HAND_LEFT;
}

/*****************************************************/
/*													 */
/*	Put inventory object in hand					 */
/*												     */
/*****************************************************/
void PutInvObjectInHand()
{
    u8 index = _index;
    u8 objectId = gPlayer.inventory[index];

    // Can put object in hand
    if (objectId <= HANDLE_OBJECTS)
    {
        u8 hand = _handX;

        if (objectId != NOTHING)
            hand = GetHandForObject(objectId);

        FlipInventoryHand(hand, objectId);
    }
    else
    {
        UseObjectInHand(index);
    }
}

/*****************************************************/
/*													 */
/*	Set selection					  				 */
/*												     */
/*****************************************************/
BOOL SetSelection(u8 move)
{
    BOOL ret = FALSE;

    WaitForSelectReleased();
    DrawSelection(UNSELECTED);
    
    if (move == FIRE)
    {
        if (_selHand)
        {
            UseObjectInHand(_handX);
            DrawHandSelection(UNSELECTED);
        }
        else
        {
            if (!UseObjectInInventory(_index))
                PutInvObjectInHand();

            DrawInventorySlot(_index);
            DrawSelection(UNSELECTED);
        }
        ret = TRUE;

        WaitForFireReleased();
    }
    else if (move < SELECT)
    {
        u8 invX = _invX;
        u8 invY = _invY;

        if (!_selHand)
        {
            if (move == TOP && invY > 0)
                _invY--;
            else
            if (move == DOWN && invY < MAX_INV_Y)
                _invY++;
            else
            if (move == LEFT && invX > 0)
                _invX--;
            else
            if (move == RIGHT && invX < MAX_INV_X)
                _invX++;

            _index = _invY * (MAX_INV_X + 1) + _invX;

            DrawSelection(SELECTED);
        }
        else
        {
            DrawHandSelection(UNSELECTED);

            if (move == LEFT)
                _handX = HAND_LEFT;
            else
            if (move == RIGHT)
                _handX = HAND_RIGHT;

            DrawHandSelection(SELECTED);
        }

        WaitForKeyReleased(move);
    }
    else
    {
        if (!_selHand || move == SUICIDE)
        {
            DrawSelection(UNSELECTED);
            ret = TRUE;
        }
        
        if (_selHand || move == SUICIDE)
        {
            DrawHandSelection(UNSELECTED);
            _selHand = FALSE;
            _index = 0;
            DrawSelection(SELECTED);
        }
    }

    if (ret)
    {
        CloseBoard();
        return FALSE;
    }

    return TRUE;
}

/*****************************************************/
/*													 */
/*	Redraw inventory				  				 */
/*												     */
/*****************************************************/
void RedrawInventory()
{
    u8 i;
    for (i = 0; i < MAX_INVENTORY; i++)
        DrawInventorySlot(i);
}

/*****************************************************/
/*													 */
/*	Draw life						  				 */
/*												     */
/*****************************************************/
void DrawLife()
{
    PrintInt(INFO_LIFE_X, INFO_LIFE_Y, gPlayer.life, 3);
}

/*****************************************************/
/*													 */
/*	Draw Gold						  				 */
/*												     */
/*****************************************************/
void DrawGold()
{
    PrintInt(INFO_GOLD_X, INFO_GOLD_Y, gPlayer.gold, 4);
}

/*****************************************************/
/*													 */
/*	Light torch inventory			  				 */
/*												     */
/*****************************************************/
void LightTorchInv()
{
    #define FLAME_POS		(RIGHT_HAND_X + 2)

    if (gTorchHand.lighted)
    {
        u8* vmem = cpctm_screenPtr(CPCT_VMEM_START, FLAME_POS, HAND_Y);
        DrawSpriteMasked(ui_flame, vmem, UI_FLAME_W, UI_FLAME_H);

        if (gTorchHand.burnout == 0)
            gTorchHand.burnout = TORCH_DURATION;
    }
}
