//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** CPCTelera includes */
#include <cpctelera.h>

/** Game includes */
#include "../structures/structures.h"
#include "../text/text.h"
#include "../text/fontsHelper.h"
#include "../sound/sound.h"
#include "../video/video.h"
#include "../input/input.h"
#include "../player/player.h"
#include "../room/room.h"
#include "../ui/uiBoard.h"

/** Sprites includes */
#include "../sprites/functions/packedSprites.h"
#include "../sprites/allSprites.h"

/** Defines */
#define POS_SHOP_X		(SCREEN_CX - TABLE_W * 2) / 2
#define POS_SHOP_Y		(ROOM_CY - TABLE_H) / 2

#define POS_ITEM_X		(SCREEN_CX - OBJECTS_W) / 2
#define POS_ITEM_Y		POS_SHOP_Y + 4

#define POS_SEL_X		POS_SHOP_X + (TABLE_W * 2 - UI_SELECTION_W) / 2
#define POS_SEL_Y		POS_SHOP_Y + TABLE_H

#define PRICE_CX		(FONT_CX_NUMBER * 4)
#define PRICE_CY		FONT_CY

#define POS_PRICE_X		(SCREEN_CX - PRICE_CX) / 2
#define POS_PRICE_Y		POS_SHOP_Y - 14

#define FIRST_OBJECT	0
#define LAST_OBJECT		4
#define NO_OBJECT		255

/** Local variable */
static u8 _selObject;

static const u8 _objects[] = { TORCH, LIGHTER, ROPE, CROWBAR, POTION };
static const u8 _price[] = { 50, 75, 90, 100, 100 };

/** Global variables */
extern volatile BOOL gDrawRoomInProgress;

/*****************************************************/
/*													 */
/*	Get object price				  				 */
/*												     */
/*****************************************************/
u8 GetPrice()
{
    return _price[_selObject];
}

/*****************************************************/
/*													 */
/*	Draw Shop Content				  				 */
/*												     */
/*****************************************************/
void DrawShopObject()
{
    /** Draw Object */
    u8 objectId = _objects[_selObject];
    u8* vmem = cpctm_screenPtr(CPCT_VMEM_START, POS_ITEM_X - 1, POS_ITEM_Y - 1);
    cpct_drawSolidBox(vmem, cpctm_px2byteM0(COLOR_WHITE, COLOR_WHITE), OBJECTS_W + 2, OBJECTS_H + 2);

    vmem = cpctm_screenPtr(CPCT_VMEM_START, POS_ITEM_X, POS_ITEM_Y);
    DrawPackedSpriteMaskedFlip(G_OBJECTS_00_INDEX, vmem, OBJECTS_W, OBJECTS_H, FALSE);
    DrawPackedSpriteMaskedFlip(G_objects_tileset[objectId], vmem, OBJECTS_W, OBJECTS_H, FALSE);

    /** Draw Price */
    vmem = cpctm_screenPtr(CPCT_VMEM_START, POS_PRICE_X, POS_PRICE_Y - 1);
    cpct_drawSolidBox(vmem, cpctm_px2byteM0(COLOR_WHITE, COLOR_WHITE), PRICE_CX, PRICE_CY + 2);
    PrintInt(POS_PRICE_X - 1, POS_PRICE_Y, GetPrice(), 3);
   
    /** Draw Item description */
    PrintTextId(TEXT_SHOP_ITEMS + _selObject);
}

/*****************************************************/
/*													 */
/*	Draw exit logo				  					 */
/*												     */
/*****************************************************/
void DrawExit(u8 spriteId)
{
    u8* vmem = cpctm_screenPtr(CPCT_VMEM_START, POS_SEL_X, POS_SEL_Y + 14);
    DrawPackedSpriteMaskedFlip(G_ui_selection_tileset[spriteId], vmem, UI_SELECTION_W, UI_SELECTION_H, FALSE);
}

/*****************************************************/
/*													 */
/*	Shop action					  					 */
/*												     */
/*****************************************************/
void ShopAction()
{
    u8 move = CheckUserInput();
    u8 prevMove = NOTHING;

    if (move == FIRE)
        WaitForFireReleased();

    move = NOTHING;
    while (move != FIRE)
    {
        move = CheckUserInput();
        if (move != NOTHING && move != prevMove)
        {
            if (move == RIGHT)
            {
                if (_selObject < LAST_OBJECT)
                    _selObject++;
                else
                    _selObject = FIRST_OBJECT;

                PlayFootStep();
            }
            else
            if (move == LEFT)
            {
                if (_selObject > FIRST_OBJECT)
                    _selObject--;
                else
                    _selObject = LAST_OBJECT;

                PlayFootStep();
            }
            else
            if (move == DOWN)
            {
                DrawExit(1);
                _selObject = NO_OBJECT;
            }
            else 
            if (move == FIRE)
            {
                u8 selObject = _selObject;
                if (selObject != NO_OBJECT)
                {
                    u8 price = GetPrice();
                    if (gPlayer.gold >= price)
                    {
                        i8 slot = PutInInventory(_objects[selObject]);
                        if (slot != NOT_FREE_SLOT)
                        {
                            gPlayer.gold -= price;
                            DrawInventorySlot(slot);
                            DrawGold();
                            PlaySelect();
                        }
                    }

                    move = SELECT;
                }
            
                WaitForFireReleased();				
            }
            else
                _selObject = FIRST_OBJECT;

            if (_selObject != NO_OBJECT)
            {
                if (_selObject > LAST_OBJECT)
                    _selObject = FIRST_OBJECT;

                DrawShopObject();
                DrawExit(2);
            }
        }

        prevMove = move;
    } 
}

/*****************************************************/
/*													 */
/*	Fill Shop background			  				 */
/*												     */
/*****************************************************/
void FillBackground()
{
    u8* backGround = GetPackedSprite(G_SHOP_GRADIENT_INDEX);
    u8 cy = 86 / SHOP_GRADIENT_H;

    u8 i;
    for (i = 0; i < cy; i++)
    {
        u8* vmem = GetScreenPtr(0, POS_SHOP_Y + i * SHOP_GRADIENT_H - 29);
        DrawMirrorCompressedSpriteMasked(backGround, vmem, SHOP_GRADIENT_W, SHOP_GRADIENT_H);

        if (i == 0 || i == cy - 1)
        {
            cpct_memset(vmem + 4, COLOR_BLACK_BYTE, SCREEN_CX - 8);
            vmem = GetScreenPtr(4, POS_SHOP_Y + i * SHOP_GRADIENT_H - 28);
            cpct_memset(vmem, COLOR_BLACK_BYTE, SCREEN_CX - 8);
        }
    }
}

/*****************************************************/
/*													 */
/*	Draw Shop						  				 */
/*												     */
/*****************************************************/
void DrawShop()
{
    /** Draw Shop */
    u8* vmem = cpctm_screenPtr(CPCT_VMEM_START, POS_SHOP_X, POS_SHOP_Y);

    /** Hide draw */
    gDrawRoomInProgress = TRUE;

    /** Drawbackground */
    CleanRoom(ROOM_CY);
    FillBackground();
    
    /** Draw Table */
    DrawMirrorPackedSpriteMasked(G_TABLE_INDEX, vmem, TABLE_W, TABLE_H);
    DrawExit(2);

    /** Draw Shop content */
    _selObject = FIRST_OBJECT;
    DrawShopObject();

    /** Show draw */
    gDrawRoomInProgress = FALSE;

    /** Take objects */
    ShopAction();
}