//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** CPCTelera includes */
#include <cpctelera.h>
#include <stdio.h>

/** Structures includes */
#include "../structures/structures.h"
#include "../structures/memMapping.h"

/** Sprite includes */
#include "../sprites/external/boardSpritesSize.h"

/** Game includes */
#include "../video/video.h"
#include "../room/room.h"
#include "../tools/tools.h"

/** World includes */
#include "./world.h"

/** World definition */
SWorld* gWorld;
SWorldStatic* gWorldStatic;

#ifdef WINCPCTELERA

static SWorld sWorld;
static SWorldStatic sWorldStatic;
static u8 sRoomsItems = 0;

/*****************************************************/
/*												     */
/*	Create Doors									 */
/*												     */
/*****************************************************/
void CreateRoom(SRoom* room, SRoomStatic* roomStatic, u8 sizeRoomId, u8 decoTorch)
{
    roomStatic->size_roomId = sizeRoomId;
    room->type_light_torch = decoTorch;
}

/*****************************************************/
/*												     */
/*	Create Room doors								 */
/*												     */
/*****************************************************/
void CreateRoomDoors(SRoom* rooms, SRoomStatic* roomStatic, u8 topDoorDef, u8 topToRoom, u8 leftDoorDef, u8 leftToRoom, u8 rightDoorDef, u8 rightToRoom, u8 downDoorDef, u8 downToRoom)
{
    u8* doorsDef = rooms->doorDef;
    u8* doorsDest = roomStatic->doorDest;

    doorsDef[DOOR_TOP] = topDoorDef;
    doorsDest[DOOR_TOP] = topToRoom;

    doorsDef[DOOR_LEFT] = leftDoorDef;
    doorsDest[DOOR_LEFT]= leftToRoom;

    doorsDef[DOOR_RIGHT] = rightDoorDef;
    doorsDest[DOOR_RIGHT] = rightToRoom;

    doorsDef[DOOR_DOWN] = downDoorDef;
    doorsDest[DOOR_DOWN] = downToRoom;
}

/*****************************************************/
/*												     */
/*	Create Room doors								 */
/*												     */
/*****************************************************/
void CreateRoomWallItems(SRoom* rooms, SRoomStatic* roomStatic, u8 decoLeft, u8 decoCenter1, u8 decoCenter2, u8 decoCenter3, u8 decoRight)
{
    u8* wallFront = rooms->wallFront;
    u8* wallLatt = roomStatic->wallLatt;

    wallLatt[DECO_LAT_LEFT] = decoLeft;
    wallFront[DECO_CENTER_1] = decoCenter1;
    wallFront[DECO_CENTER_2] = decoCenter2;
    wallFront[DECO_CENTER_3] = decoCenter3;
    wallLatt[DECO_LAT_RIGHT] = decoRight;
}

/*****************************************************/
/*												     */
/*	Create Room doors								 */
/*												     */
/*****************************************************/
void CreateRoomItems(SRoom* rooms, SRoomStatic* roomStatic, u8 roomX, u8 roomY, u8 item)
{
    u8 roomId = GET_ROOM_ID(roomStatic->size_roomId);

    SItemsRoom* itemRooms = GetRoomItems(roomId);
    if (itemRooms == NULL)
    {
        itemRooms = &gWorld->roomsItems[sRoomsItems++];
        itemRooms->roomId = roomId;
    }

    u8* items = &itemRooms->items[roomY * NB_ROOM_ITEMS];
    items[roomX] = item;
}

/*****************************************************/
/*												     */
/*	Create Chest									 */
/*												     */
/*****************************************************/
void CreateChest(SRoom* rooms, SRoomStatic* roomStatic, u8 lock, u8 roomX, u8 roomY, u8 itemId1, u8 itemId2, u8 itemId3)
{
    static u8 sChest = 0;	
    SChest* chest = &gWorld->chests[sChest++];

    CreateRoomItems(rooms, roomStatic, roomX, roomY, ROOM_CHEST_CLOSE);

    chest->roomId = GET_ROOM_ID(roomStatic->size_roomId);
    chest->solidity = (lock == CHEST_LOCK_LOCKED) ? 2 : 0;
    chest->items[0] = itemId1;
    chest->items[1] = itemId2;
    chest->items[2] = itemId3;
}

/*****************************************************/
/*												     */
/*	Create Rooms									 */
/*  TOP / LEFT / RIGHT / DOWN						 */
/*												     */
/*****************************************************/
void CreateRooms()
{
    SRoom* rooms = gWorld->rooms;
    SRoomStatic* roomsStatic = gWorldStatic->roomsStatic;
    u8 roomId = 0;

    // Forest
    /** Room 0 */
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, ROOM_LIGHT | ROOM_FOREST);
    CreateRoomWallItems(rooms, roomsStatic, NO_DECO, WALL_EMPTY, WALL_EMPTY, WALL_FOREST_SIGN, NO_DECO);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_NO_WALL, 1,					// LEFT
        DOOR_NO_WALL, SHOP_ROOM,			// RIGHT	
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 1 */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, ROOM_LIGHT | ROOM_FOREST);
    CreateRoomWallItems(rooms, roomsStatic, NO_DECO, WALL_FOREST_SIGN, WALL_EMPTY, WALL_EMPTY, NO_DECO);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_NO_WALL, 2,					// LEFT
        DOOR_NO_WALL, 0,					// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 2 to Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, ROOM_LIGHT | ROOM_FOREST);
    CreateRoomWallItems(rooms, roomsStatic, NO_DECO, WALL_FOREST_SIGN, WALL_EMPTY, WALL_EMPTY, NO_DECO);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_PASSAGE, 5,					// TOP (CASTLE)
        DOOR_NO_WALL, LAC_FOREST_ROOM,	// LEFT (LAC_FOREST : 3)
        DOOR_NO_WALL, 1,					// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 3 Lac */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, ROOM_LIGHT | ROOM_FOREST);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_WALL, NOTHING,				// LEFT
        DOOR_NO_WALL, TO_CASTLE_ROOM,	// RIGHT (TO_CASTLE_ROOM : 2)
        DOOR_WALL, NOTHING);				// DOWN
                            
    /** Room 4 */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, ROOM_LIGHT | TORCH_RIGHT_EMPTY);
    CreateRoomWallItems(rooms, roomsStatic, WALL_WINDOW, NO_DECO, WALL_SHIELD_0, NO_DECO, NO_DECO);
    CreateChest(rooms, roomsStatic, CHEST_LOCK_OPEN, 0, 0, TORCH, SCROLL_0, GOLD_PURSE);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_SECRET, 60,					// TOP
        DOOR_WALL, NOTHING,					// LEFT
        DOOR_PASSAGE, 5,					// RIGHT
        DOOR_WALL, NOTHING);				// DOWN									
                                    
    // Castle Floor
    /** Room 5 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, TORCH_RIGHT_EMPTY | TORCH_LEFT_EXTING | ROOM_LIGHT);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WOOD_METAL_KEY, 11,		    // TOP
        DOOR_PASSAGE, 4,					// LEFT
        DOOR_PASSAGE, 6,					// RIGHT
        DOOR_WOOD_OPENED, TO_CASTLE_ROOM);	// DOWN (TO_CASTLE_ROOM : 3)

    /** Room 6 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_CORRIDOR_MASK | roomId++, ROOM_LIGHT | TORCH_LEFT_EXTING);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WOOD_OPENED, 10,				// TOP
        DOOR_PASSAGE, 5,					// LEFT
        DOOR_NO_WALL, 7,					// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 7 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_CORRIDOR_MASK | roomId++, ROOM_LIGHT | TORCH_RIGHT_EXTING);
    CreateRoomWallItems(rooms, roomsStatic, NO_DECO, NO_DECO, WALL_PAINT_2, NO_DECO, NO_DECO);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_NO_WALL, 6,					// LEFT
        DOOR_WOOD_METAL_KEY, 8,					// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 8 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, ROOM_LIGHT | TORCH_LEFT_EMPTY);
    CreateRoomWallItems(rooms, roomsStatic, WALL_TAPESTRY_BLUE, NO_DECO, NO_DECO, NO_DECO, WALL_TAPESTRY_BLUE);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_NO_WALL, 13,				// TOP
        DOOR_WOOD_METAL_KEY, 7,					// LEFT
        DOOR_PASSAGE, 9,					// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 9 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, ROOM_LIGHT | TORCH_LEFT_EXTING);
    CreateRoomWallItems(rooms, roomsStatic, NO_DECO, NO_DECO, NO_DECO, NO_DECO, WALL_WINDOW);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_STAIR_DOWN, 20,	// TOP
        DOOR_PASSAGE, 8,					// LEFT
        DOOR_WALL, NOTHING,				// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 10 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, TORCH_RIGHT_EXTING);
    CreateRoomItems(rooms, roomsStatic, 1, 2, SCROLL_2);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_STAIR_DOWN, 25,				// TOP
        DOOR_WALL, NOTHING,				// LEFT
        DOOR_WALL, NOTHING,				// RIGHT
        DOOR_WOOD_OPENED, 6);				// DOWN

    /** Room 11 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, ROOM_LIGHT | TORCH_LEFT_EXTING);
    CreateRoomWallItems(rooms, roomsStatic, WALL_WINDOW, NO_DECO, NO_DECO, NO_DECO, WALL_TAPESTRY_ORANGE);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_NO_WALL, 14,				// TOP
        DOOR_WALL, NOTHING,				// LEFT
        DOOR_WALL, NOTHING,				// RIGHT
        DOOR_WOOD_METAL_KEY, 5);				// DOWN

    /** Room 12 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, TORCH_RIGHT_EXTING);
    CreateRoomWallItems(rooms, roomsStatic, NO_DECO, NO_DECO, WALL_FIREPLACE, NO_DECO, NO_DECO);
    CreateRoomItems(rooms, roomsStatic, 0, 2, GOLD_PURSE);
    CreateRoomItems(rooms, roomsStatic, 1, 2, ROOM_ITEM_HIDE_TRAP);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_WALL, NOTHING,				// LEFT
        DOOR_PASSAGE, 13,				// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 13 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, ROOM_LIGHT | TORCH_LEFT_EXTING);
    CreateRoomWallItems(rooms, roomsStatic, NO_DECO, NO_DECO, NO_DECO, NO_DECO, WALL_WINDOW);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_PASSAGE, 17,				// TOP
        DOOR_PASSAGE, 12,				// LEFT
        DOOR_WALL, NOTHING,				// RIGHT
        DOOR_NO_WALL, 8);				// DOWN

    /** Room 14 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, ROOM_LIGHT | TORCH_RIGHT_EXTING);
    CreateRoomWallItems(rooms, roomsStatic, WALL_WINDOW, NO_DECO, NO_DECO, NO_DECO, NO_DECO);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_PASSAGE, 18,				// TOP
        DOOR_WALL, NOTHING,				// LEFT
        DOOR_WOOD_OPENED, 15,				// RIGHT
        DOOR_NO_WALL, 11);				// DOWN

    /** Room 15 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_CORRIDOR_MASK | roomId++, ROOM_LIGHT | TORCH_LEFT_EXTING);
    CreateRoomWallItems(rooms, roomsStatic, NO_DECO, WALL_WINDOW, NO_DECO, WALL_WINDOW, NO_DECO);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_WOOD_OPENED, 14,				// LEFT
        DOOR_NO_WALL, 16,				// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 16 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_CORRIDOR_MASK | roomId++, ROOM_LIGHT | TORCH_RIGHT_EMPTY);
    CreateRoomWallItems(rooms, roomsStatic, NO_DECO, WALL_WINDOW, WALL_PAINT_2, WALL_WINDOW, NO_DECO);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_NO_WALL, 15,				// LEFT
        DOOR_WOOD_METAL_KEY, 17,				// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 17 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, ROOM_LIGHT | TORCH_LEFT_EXTING);
    CreateRoomWallItems(rooms, roomsStatic, NO_DECO, NO_DECO, NO_DECO, NO_DECO, WALL_WINDOW);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_PASSAGE, 19,				// TOP
        DOOR_WOOD_METAL_KEY, 16,				// LEFT
        DOOR_WOOD_CLOSED, 37,				// RIGHT
        DOOR_PASSAGE, 13);				// DOWN

    /** Room 18 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, ROOM_LIGHT | TORCH_RIGHT_EXTING);
    CreateRoomWallItems(rooms, roomsStatic, WALL_WINDOW, NO_DECO, NO_DECO, NO_DECO, WALL_WINDOW);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_STAIR_UP, 30,				// TOP
        DOOR_WALL, NOTHING,				// LEFT
        DOOR_WALL, NOTHING,				// RIGHT
        DOOR_PASSAGE, 14);				// DOWN

    /** Room 19 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, ROOM_LIGHT | TORCH_LEFT_EXTING);
    CreateRoomWallItems(rooms, roomsStatic, WALL_WINDOW, NO_DECO, NO_DECO, NO_DECO, WALL_WINDOW);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_STAIR_UP, 40,				// TOP
        DOOR_WALL, NOTHING,				// LEFT
        DOOR_WALL, NOTHING,				// RIGHT
        DOOR_PASSAGE, 17);				// DOWN

    // Prisons

    /** Room 20 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++,  TORCH_LEFT_EXTING);
    CreateRoomWallItems(rooms, roomsStatic, NO_DECO, NO_DECO, NO_DECO, NO_DECO, WALL_TAPESTRY_BLUE);
    CreateRoomItems(rooms, roomsStatic, 0, 0, ROOM_TABLE);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, 0,				// TOP
        DOOR_PASSAGE, 21,				// LEFT
        DOOR_WALL, NOTHING,				// RIGHT
        DOOR_PASSAGE, 9);				// DOWN


    /** Room 21 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_CORRIDOR_MASK | roomId++,  TORCH_RIGHT_EXTING);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WOOD_PRISON_CLOSED, 23,				// TOP
        DOOR_NO_WALL, 22,				// LEFT
        DOOR_PASSAGE, 20,				// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 22 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_CORRIDOR_MASK | roomId++,  TORCH_LEFT_EXTING);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WOOD_PRISON_CLOSED, 24,				// TOP
        DOOR_WALL, NOTHING,				// LEFT
        DOOR_NO_WALL, 21,				// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 23 Castle Cells */
    /** KING CELL */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, NOTHING);
    CreateRoomWallItems(rooms, roomsStatic, NO_DECO, WALL_CHAINS, WALL_CHAINS, WALL_CHAINS, NO_DECO);
    CreateRoomItems(rooms, roomsStatic, 0, 1, ROOM_SKELETON);
    CreateRoomItems(rooms, roomsStatic, 2, 1, RING);

    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_WALL, NOTHING,				// LEFT
        DOOR_WALL, NOTHING,				// RIGHT
        DOOR_WOOD_PRISON_CLOSED, 21);				// DOWN

    /** Room 24 Castle Cells */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, NOTHING);
    CreateRoomWallItems(rooms, roomsStatic, NO_DECO, WALL_CHAINS, WALL_CHAINS, WALL_CHAINS, NO_DECO);
    CreateRoomItems(rooms, roomsStatic, 2, 2, ROOM_SKELETON);
    CreateRoomItems(rooms, roomsStatic, 1, 2, SCROLL_1);

    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_WALL, NOTHING,				// LEFT
        DOOR_WALL, NOTHING,				// RIGHT
        DOOR_WOOD_PRISON_CLOSED, 22);				// DOWN

    /** Room 25 Castle Level -1 */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, TORCH_LEFT_EXTING);
    CreateChest(rooms, roomsStatic, CHEST_LOCK_OPEN, 0, 0, NOTHING, NOTHING, GOLD_PURSE);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_WOOD_OPENED, 26,			// LEFT
        DOOR_WALL, NOTHING,				// RIGHT
        DOOR_PASSAGE, 10);				// DOWN

    /** Room 26 Castle Level -1 */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_CORRIDOR_MASK | roomId++, NO_LIGHT | TORCH_RIGHT_EXTING);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_NO_WALL, 27,				// LEFT
        DOOR_WOOD_OPENED, 25,				// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 27 Castle Level -1 */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_CORRIDOR_MASK | roomId++, NO_LIGHT | TORCH_LEFT_EMPTY);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WOOD_CLOSED, 28,				// TOP
        DOOR_WALL, NOTHING,				// LEFT
        DOOR_NO_WALL, 26,				// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 28 Castle Level -1 */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, TORCH_RIGHT_EXTING);
    CreateRoomWallItems(rooms, roomsStatic, NO_DECO, NO_DECO, WALL_FIREPLACE_KEY, NO_DECO, NO_DECO);
    CreateRoomItems(rooms, roomsStatic, 2, 1, GOLD_PURSE);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_WALL, NOTHING,				// LEFT
        DOOR_WALL, NOTHING,				// RIGHT
        DOOR_WOOD_CLOSED, 27);				// DOWN

    /** Room 29 - Win game */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, TORCH_LEFT_BURNISH | TORCH_RIGHT_BURNISH);
    CreateRoomWallItems(rooms, roomsStatic, WALL_WINDOW, WALL_ALTAR_SCEPTER, WALL_ALTAR, WALL_ALTAR_RING, WALL_WINDOW);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_WALL, NOTHING,				// LEFT
        DOOR_WALL, NOTHING,				// RIGHT
        DOOR_NO_WALL, NOTHING);				// DOWN

    /** Room 30 Castle - Level 2 */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, TORCH_RIGHT_EXTING);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_PASSAGE, 32,				// TOP
        DOOR_WALL, NOTHING,				// LEFT
        DOOR_WALL, NOTHING,				// RIGHT
        DOOR_PASSAGE, 18);				// DOWN

    /** Room 31 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_CORRIDOR_MASK | roomId++, TORCH_LEFT_EXTING);
    CreateRoomWallItems(rooms, roomsStatic, NO_DECO, WALL_EMPTY, WALL_EMPTY, WALL_PAINT_3, NO_DECO);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_PASSAGE, 34,				// TOP
        DOOR_WALL, NOTHING,				// LEFT
        DOOR_NO_WALL, 32,				// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 32 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_CORRIDOR_MASK | roomId++, TORCH_RIGHT_EXTING);
    CreateRoomWallItems(rooms, roomsStatic, NO_DECO, WALL_PAINT_0, NO_DECO, WALL_PAINT_1, NO_DECO);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_NO_WALL, 31,				// LEFT
        DOOR_PASSAGE, 33,				// RIGHT
        DOOR_PASSAGE, 30);				// DOWN

    /** Room 33 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, TORCH_LEFT_EXTING);
    CreateRoomWallItems(rooms, roomsStatic, NO_DECO, NO_DECO, WALL_PAINT_0, NO_DECO, NO_DECO);
    CreateRoomItems(rooms, roomsStatic, 0, 0, GOLD_PURSE);
    CreateRoomItems(rooms, roomsStatic, 0, 1, ROOM_ITEM_HIDE_TRAP);
    CreateRoomItems(rooms, roomsStatic, 1, 2, ROOM_ITEM_HIDE_TRAP);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_PASSAGE, 32,					// LEFT
        DOOR_WALL, NOTHING,				// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 34 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, TORCH_RIGHT_EXTING);
    CreateRoomWallItems(rooms, roomsStatic, WALL_SECRET_KEY_HOLE, NO_DECO, NO_DECO, NO_DECO, NO_DECO);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_SECRET, 35,	// TOP
        DOOR_WALL, NOTHING,				// LEFT
        DOOR_WALL, NOTHING,				// RIGHT
        DOOR_PASSAGE, 31);				// DOWN

    /** Room 35 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, TORCH_RIGHT_EXTING);
    CreateRoomWallItems(rooms, roomsStatic, NO_DECO, NO_DECO, WALL_SHIELD_0, NO_DECO, NO_DECO);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_SECRET_OPENED, 34,				// TOP
        DOOR_WALL, NOTHING,				// LEFT
        DOOR_WOOD_OPENED, 36,					// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 36 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, TORCH_LEFT_EXTING);
    CreateRoomWallItems(rooms, roomsStatic, WALL_TAPESTRY_ORANGE, WALL_TAPESTRY_ORANGE, WALL_ALTAR_CROWN, WALL_TAPESTRY_ORANGE, WALL_TAPESTRY_ORANGE);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_WOOD_OPENED, 35,				// LEFT
        DOOR_WALL, NOTHING,					// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 37 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, ROOM_LIGHT | ROOM_FOREST);
    CreateRoomWallItems(rooms, roomsStatic, NO_DECO, WALL_FOREST_SIGN, WALL_EMPTY, WALL_FOREST_SIGN, NO_DECO);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_PASSAGE, 17,				// LEFT
        DOOR_PASSAGE, 38,					// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 38 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, ROOM_LIGHT);
    CreateRoomWallItems(rooms, roomsStatic, WALL_WINDOW, WALL_TAPESTRY_BLUE, WALL_ALTAR_GOLD_CUP, WALL_TAPESTRY_BLUE, WALL_WINDOW);
    CreateChest(rooms, roomsStatic, CHEST_LOCK_LOCKED, 0, 0, NOTHING, GOLD_PURSE, NOTHING);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_WOOD_OPENED, 37,				// LEFT
        DOOR_WALL, NOTHING,					// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 39 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_PIT_MASK | roomId++, ROOM_LIGHT | TORCH_LEFT_EMPTY);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, 56,					// TOP
        DOOR_WALL, NOTHING,					// LEFT
        DOOR_WALL, NOTHING,					// RIGHT
        DOOR_WOOD_OPENED, 41);				// DOWN

    /** Room 40 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, ROOM_LIGHT | TORCH_LEFT_EMPTY);
    CreateRoomWallItems(rooms, roomsStatic, NOTHING, NOTHING, WALL_TAPESTRY_RED, NOTHING, WALL_WINDOW);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_PASSAGE, 44,					// LEFT
        DOOR_WALL, NOTHING,					// RIGHT
        DOOR_PASSAGE, 19);				// DOWN

    /** Room 41 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, ROOM_LIGHT);
    CreateRoomWallItems(rooms, roomsStatic, WALL_WINDOW, NOTHING, NOTHING, NOTHING, NOTHING);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_PASSAGE, 39,				// TOP
        DOOR_WALL, NOTHING,					// LEFT
        DOOR_WALL, NOTHING,					// RIGHT
        DOOR_WOOD_METAL_KEY, 45);				// DOWN

    /** Room 42 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, TORCH_RIGHT_EMPTY);
    CreateRoomWallItems(rooms, roomsStatic, NOTHING, NOTHING, WALL_PAINT_3, NOTHING, NOTHING);
    CreateRoomItems(rooms, roomsStatic, 0, 0, ROOM_SKELETON);
    CreateRoomItems(rooms, roomsStatic, 2, 0, GOLD_PURSE);
    CreateRoomItems(rooms, roomsStatic, 1, 2, ROOM_EVENT_LIGHT_OFF);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_WALL, NOTHING,					// LEFT
        DOOR_PASSAGE, 43,					// RIGHT
        DOOR_PASSAGE, 46);				// DOWN

    /** Room 43 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_CORRIDOR_MASK | roomId++, TORCH_LEFT_EMPTY);
    CreateRoomItems(rooms, roomsStatic, 0, 2, ROOM_DEMON_CIRCLE);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_PASSAGE, 44,				// TOP
        DOOR_PASSAGE, 42,					// LEFT
        DOOR_PASSAGE, 55,					// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 44 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, TORCH_RIGHT_EMPTY);
    CreateRoomWallItems(rooms, roomsStatic, NOTHING, NOTHING, WALL_PAINT_2, NOTHING, NOTHING);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_WALL, NOTHING,					// LEFT
        DOOR_PASSAGE, 40,					// RIGHT
        DOOR_PASSAGE, 43);				// DOWN

    /** Room 45 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, TORCH_LEFT_EMPTY);
    CreateRoomWallItems(rooms, roomsStatic, NOTHING, WALL_TAPESTRY_RED, NOTHING, NOTHING, WALL_TAPESTRY_RED);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WOOD_METAL_KEY, 41,				// TOP
        DOOR_WALL, NOTHING,					// LEFT
        DOOR_WALL, NOTHING,					// RIGHT
        DOOR_PASSAGE, 50);				// DOWN

    /** Room 46 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, TORCH_LEFT_EMPTY);
    CreateRoomItems(rooms, roomsStatic, 0, 1, ROOM_SKELETON);
    CreateRoomItems(rooms, roomsStatic, 2, 2, ROOM_EVENT_LIGHT_OFF);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_PASSAGE, 42,				// TOP
        DOOR_WALL, NOTHING,					// LEFT
        DOOR_PASSAGE, 47,					// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 47 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, TORCH_RIGHT_EMPTY);
    CreateRoomWallItems(rooms, roomsStatic, WALL_TAPESTRY_RED, NOTHING, WALL_PAINT_2, NOTHING, WALL_TAPESTRY_RED);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_PASSAGE, 46,					// LEFT
        DOOR_WOOD_CLOSED, 48,					// RIGHT
        DOOR_PASSAGE, 52);				// DOWN

    /** Room 48 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, ROOM_LIGHT | TORCH_LEFT_EMPTY);
    CreateRoomWallItems(rooms, roomsStatic, NOTHING, WALL_TAPESTRY_BLUE, WALL_ALTAR, WALL_TAPESTRY_BLUE, WALL_WINDOW);
    CreateRoomItems(rooms, roomsStatic, 0, 1, ROOM_SKELETON);
    CreateRoomItems(rooms, roomsStatic, 2, 1, POTION);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_WOOD_CLOSED, 47,					// LEFT
        DOOR_WALL, NOTHING,					// RIGHT
        DOOR_WOOD_CLOSED, 53);				// DOWN

    /** Room 49 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, ROOM_LIGHT | TORCH_RIGHT_EXTING);
    CreateRoomWallItems(rooms, roomsStatic, WALL_WINDOW, NOTHING, WALL_WINDOW, NOTHING, NOTHING);
    CreateChest(rooms, roomsStatic, CHEST_LOCK_LOCKED, 0, 0, GOLD_PURSE, TORCH, NOTHING);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_WALL, NOTHING,					// LEFT
        DOOR_WOOD_OPENED, 50,					// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 50 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, TORCH_LEFT_EXTING | TORCH_RIGHT_EMPTY);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_PASSAGE, 45,				// TOP
        DOOR_WOOD_OPENED, 49,					// LEFT
        DOOR_WOOD_METAL_KEY, 51,					// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 51 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_CORRIDOR_MASK | roomId++, TORCH_LEFT_EMPTY | TORCH_RIGHT_EMPTY);
    CreateRoomWallItems(rooms, roomsStatic, NOTHING, WALL_PAINT_1, WALL_PAINT_0, NOTHING, NOTHING);
    CreateRoomItems(rooms, roomsStatic, 1, 2, ROOM_ITEM_HIDE_TRAP);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_WOOD_METAL_KEY, 50,					// LEFT
        DOOR_PASSAGE, 52,					// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 52 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_CORRIDOR_MASK | roomId++, TORCH_LEFT_EMPTY | TORCH_RIGHT_EMPTY);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_PASSAGE, 47,				// TOP
        DOOR_PASSAGE, 51,					// LEFT
        DOOR_PASSAGE, 53,					// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 53 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_CORRIDOR_MASK | roomId++, TORCH_LEFT_EMPTY | TORCH_RIGHT_EMPTY);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WOOD_CLOSED, 48,				// TOP
        DOOR_PASSAGE, 52,					// LEFT
        DOOR_WOOD_OPENED, 54,					// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 54 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, TORCH_LEFT_EMPTY);
    CreateRoomWallItems(rooms, roomsStatic, NOTHING, NOTHING, WALL_FIREPLACE, NOTHING, NOTHING);
    CreateChest(rooms, roomsStatic, CHEST_LOCK_LOCKED, 0, 0, GOLD_KEY, TORCH, NOTHING);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_WOOD_OPENED, 53,					// LEFT
        DOOR_WALL, NOTHING,					// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 55 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, ROOM_LIGHT | TORCH_LEFT_EMPTY);
    CreateRoomWallItems(rooms, roomsStatic, NO_DECO, WALL_FIREPLACE, NO_DECO, NO_DECO, WALL_WINDOW);
    CreateRoomItems(rooms, roomsStatic, 2, 0, TORCH);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_PASSAGE, 43,					// LEFT
        DOOR_WALL, NOTHING,					// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 56 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, TORCH_LEFT_EMPTY);
    CreateRoomItems(rooms, roomsStatic, 1, 1, ROOM_ITEM_TRAP);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_PASSAGE, 58,				// TOP
        DOOR_WALL, NOTHING,					// LEFT
        DOOR_WALL, NOTHING,					// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 57 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_CORRIDOR_MASK | roomId++, TORCH_LEFT_EMPTY);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_PASSAGE, 60,				// TOP
        DOOR_WALL, NOTHING,					// LEFT
        DOOR_NO_WALL, 58,					// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 58 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_CORRIDOR_MASK | roomId++, NO_LIGHT);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WOOD_OPENED, 61,				// TOP
        DOOR_NO_WALL, 57,					// LEFT
        DOOR_NO_WALL, 59,					// RIGHT
        DOOR_PASSAGE, 56);				// DOWN

    /** Room 59 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_CORRIDOR_MASK | roomId++, TORCH_RIGHT_EXTING);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WOOD_CLOSED, 62,				// TOP
        DOOR_NO_WALL, 58,					// LEFT
        DOOR_WALL, NOTHING,					// RIGHT
        DOOR_WALL, NOTHING);				// DOWN

    /** Room 60 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, TORCH_LEFT_EXTING);
    CreateRoomWallItems(rooms, roomsStatic, WALL_SECRET_KEY_HOLE, NO_DECO, NO_DECO, NO_DECO, NO_DECO);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_SECRET, 4,					// TOP
        DOOR_WALL, NOTHING,				// LEFT
        DOOR_WALL, NOTHING,				// RIGHT
        DOOR_PASSAGE, 57);				// DOWN

    /** Room 61 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, TORCH_RIGHT_EMPTY);
    CreateRoomWallItems(rooms, roomsStatic, WALL_TAPESTRY_RED, WALL_PAINT_0, NOTHING, NOTHING, NOTHING);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_WALL, NOTHING,				// LEFT
        DOOR_WOOD_CLOSED, 62,				// RIGHT
        DOOR_WOOD_OPENED, 58);				// DOWN
    
    /** Room 62 Castle */
    rooms++; roomsStatic++;
    CreateRoom(rooms, roomsStatic, ROOM_LARGE_MASK | roomId++, TORCH_LEFT_EMPTY);
    CreateRoomWallItems(rooms, roomsStatic, WALL_TAPESTRY_RED, WALL_TAPESTRY_RED, NOTHING, WALL_TAPESTRY_RED, WALL_TAPESTRY_RED);
    CreateChest(rooms, roomsStatic, CHEST_LOCK_LOCKED, 0, 0, SCEPTER, GOLD_PURSE, NOTHING);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, NOTHING,				// TOP
        DOOR_WOOD_CLOSED, 61,				// LEFT
        DOOR_WALL, NOTHING,				// RIGHT
        DOOR_WOOD_CLOSED, 59);				// DOWN

    /****************/
    /** Room 63 Pit */
    rooms = &gWorld->rooms[PIT_ROOM];
    roomsStatic = &gWorldStatic->roomsStatic[PIT_ROOM];
    CreateRoom(rooms, roomsStatic, ROOM_PIT_MASK | PIT_ROOM, ROOM_LIGHT);
    CreateRoomDoors(rooms, roomsStatic,
        DOOR_WALL, 0,		// TOP
        DOOR_WALL, 0,		// LEFT
        DOOR_WALL, 0,	// RIGHT
        DOOR_WALL, 0);	// DOWN
}

/*****************************************************/
/*												     */
/*	Create world									 */
/*												     */
/*****************************************************/
void SaveWorld()
{
    FILE* fileWorld = NULL;
    fopen_s(&fileWorld, "../resources/dat/world.dat", "wb");
    if (fileWorld != NULL)
    {
        fwrite(gWorld, sizeof(SWorld), 1, fileWorld);
        fclose(fileWorld);
    }

    fopen_s(&fileWorld, "../resources/dat/worldsta.dat", "wb");
    if (fileWorld != NULL)
    {
        fwrite(gWorldStatic, sizeof(SWorldStatic), 1, fileWorld);
        fclose(fileWorld);
    }
}

/*****************************************************/
/*												     */
/*	Load world										 */
/*												     */
/*****************************************************/
void LoadWorld()
{
    gWorld = &sWorld;
    gWorldStatic = &sWorldStatic;
    CreateRooms();
    SaveWorld();
}

/*****************************************************/
/*												     */
/*	Move world to memory location					 */
/*												     */
/*****************************************************/
void InitWorld()
{
    ResetWorld();
}

#else

/*****************************************************/
/*												     */
/*	Move world to memory location					 */
/*												     */
/*****************************************************/
void InitWorld()
{
    gWorldStatic = GET_WORLD_STATIC_LOC;
    gWorld = GET_WORLD_LOC;
   
    cpct_memcpy(gWorldStatic, WORLD_STATIC_LOC_LOAD, sizeof(SWorldStatic));
    cpct_memcpy(GET_WORLD_LOC_COMPRESS, WORLD_LOC_COMP_LOAD, WORLD_COMPRESS_SIZE);

    ResetWorld();
}
#endif